var _a;

import * as tsplus_module_1 from "@tsplus/stdlib/structure/Equals";
import * as tsplus_module_2 from "@tsplus/stdlib/structure/Hash";
import * as tsplus_module_3 from "@tsplus/stdlib/data/AtomicNumber";
import * as tsplus_module_4 from "@tsplus/stdlib/exceptions";
export const isChunk = isChunk_1;
export const corresponds_ = corresponds_1;
export const BufferSize = 64;
export const ChunkTypeId = /*#__PURE__*/Symbol.for("@tsplus/collections/Chunk");
export const alloc = // @ts-ignore
typeof Buffer !== "undefined" ? Buffer.alloc : n => new Uint8Array(n);
export function isByte(u) {
  return typeof u === "number" && Number.isInteger(u) && u >= 0 && u <= 255;
}
export const Chunk = {
  $: {}
};
/**
 * @tsplus unify Chunk
 */

export function unifyChunk(self) {
  return self;
}
/**
 * Internal base class
 */

export class ChunkInternal {
  constructor() {
    this[_a] = ChunkTypeId;
  }

  _arrayLike() {
    if (this.arrayLikeCache) {
      return this.arrayLikeCache;
    }

    const arr = this.binary ? alloc(this.length) : new Array(this.length);

    this._copyToArray(0, arr);

    this.arrayLikeCache = arr;
    return arr;
  }

  _array() {
    if (this.arrayCache) {
      return this.arrayCache;
    }

    const arr = new Array(this.length);

    this._copyToArray(0, arr);

    this.arrayCache = arr;
    return arr;
  }

  [(_a = ChunkTypeId, tsplus_module_1.Equals.sym)](that) {
    return isChunk_1(that) && corresponds_1(this, that, tsplus_module_1.equals);
  }

  [tsplus_module_2.Hash.sym]() {
    return tsplus_module_2.hashIterator(this[Symbol.iterator]());
  }

  toString() {
    return `Chunk(${this._array().join(", ")})`;
  }

  toJSON() {
    return this._array();
  }

  _buckets() {
    return {
      [Symbol.iterator]: () => this._arrayLikeIterator()
    };
  }

  _reverseBuckets() {
    return {
      [Symbol.iterator]: () => this._reverseArrayLikeIterator()
    };
  }

  _reverse() {
    const arr = this._arrayLike();

    return {
      [Symbol.iterator]: () => {
        let i = arr.length - 1;
        return {
          next: () => {
            if (i >= 0 && i < arr.length) {
              const k = arr[i];
              i--;
              return {
                value: k,
                done: false
              };
            }

            return {
              value: arr.length,
              done: true
            };
          }
        };
      }
    };
  }

  _materialize() {
    void 0;

    switch (this._typeId) {
      case EmptyTypeId:
        {
          return this;
        }

      case ArrTypeId:
        {
          return this;
        }

      default:
        {
          return array_(this._arrayLike());
        }
    }
  }

  _append(a1) {
    const binary = this.binary && isByte(a1);
    const buffer = this.binary && binary ? alloc(BufferSize) : new Array(BufferSize);
    buffer[0] = a1;
    return new AppendN(this, buffer, 1, new tsplus_module_3.AtomicNumber(1), this.binary && binary);
  }

  _prepend(a1) {
    const binary = this.binary && isByte(a1);
    const buffer = this.binary && binary ? alloc(BufferSize) : new Array(BufferSize);
    buffer[BufferSize - 1] = a1;
    return new PrependN(this, buffer, 1, new tsplus_module_3.AtomicNumber(1), this.binary && binary);
  }

  _take(n) {
    void 0;

    if (n <= 0) {
      return _Empty;
    } else if (n >= this.length) {
      return this;
    } else {
      switch (this._typeId) {
        case EmptyTypeId:
          {
            return _Empty;
          }

        case SliceTypeId:
          {
            if (n >= this.length) {
              return this;
            } else {
              return new Slice(this.chunk, this.offset, n);
            }
          }

        case SingletonTypeId:
          {
            return this;
          }

        default:
          {
            return new Slice(this, 0, n);
          }
      }
    }
  }

  _concat(that) {
    void 0;
    void 0;

    if (this._typeId === EmptyTypeId) {
      return that;
    }

    if (that._typeId === EmptyTypeId) {
      return this;
    }

    if (this._typeId === AppendNTypeId) {
      const chunk = array_(this.buffer)._take(this.bufferUsed);

      return this.start._concat(chunk)._concat(that);
    }

    if (that._typeId === PrependNTypeId) {
      const chunk = array_(that.bufferUsed === 0 ? [] : that.buffer.slice(-that.bufferUsed));
      return this._concat(chunk)._concat(that.end);
    }

    const diff = that.depth - this.depth;

    if (Math.abs(diff) <= 1) {
      return new Concat(this, that);
    } else if (diff < -1) {
      if (this.left.depth >= this.right.depth) {
        const nr = this.right._concat(that);

        return new Concat(this.left, nr);
      } else {
        const nrr = this.right.right._concat(that);

        if (nrr.depth === this.depth - 3) {
          const nr = new Concat(this.right.left, nrr);
          return new Concat(this.left, nr);
        } else {
          const nl = new Concat(this.left, this.right.left);
          return new Concat(nl, nrr);
        }
      }
    } else {
      if (this.right.depth >= that.left.depth) {
        const nl = this._concat(that.left);

        return new Concat(nl, that.right);
      } else {
        const nll = this._concat(that.left.left);

        if (nll.depth === that.depth - 3) {
          const nl = new Concat(nll, that.left.right);
          return new Concat(nl, that.right);
        } else {
          const nr = new Concat(that.left.right, that.right);
          return new Concat(nll, nr);
        }
      }
    }
  }

}
export const EmptyTypeId = /*#__PURE__*/Symbol.for("@effect-ts/core/collection/immutable/Chunk/Empty");
/**
 * Internal Empty Chunk
 */

export class Empty extends ChunkInternal {
  _get(n) {
    throw new tsplus_module_4.IndexOutOfBounds(n, 0, this.length - 1);
  }

  constructor() {
    super();
    this.depth = 0;
    this._typeId = EmptyTypeId;
    this.left = this;
    this.right = this;
    this.binary = true;
    this.length = 0;
  }

  _materialize() {
    return array_([]);
  }

  _copyToArray(_n, _array) {// no-op
  }

  [Symbol.iterator]() {
    return {
      next: () => ({
        value: 0,
        done: true
      })
    };
  }

  _arrayLikeIterator() {
    return {
      next: () => ({
        value: 0,
        done: true
      })
    };
  }

  _reverseArrayLikeIterator() {
    return {
      next: () => ({
        value: 0,
        done: true
      })
    };
  }

}
export const _Empty = /*#__PURE__*/new Empty();
/**
 * @tsplus macro remove
 */

export function concreteChunk(_) {//
}
/**
 * @tsplus macro identity
 */

export function concreteChunkId(_) {
  void 0;
  return _;
}
export const AppendNTypeId = /*#__PURE__*/Symbol.for("@effect-ts/core/collection/immutable/Chunk/AppendN");
/**
 * Internal Append Chunk
 */

export class AppendN extends ChunkInternal {
  constructor(start, buffer, bufferUsed, chain, binary) {
    super();
    this.start = start;
    this.buffer = buffer;
    this.bufferUsed = bufferUsed;
    this.chain = chain;
    this.binary = binary;
    this._typeId = AppendNTypeId;
    this.depth = 0;
    this.left = _Empty;
    this.right = _Empty;
    this.length = this.start.length + this.bufferUsed;
  }

  _get(n) {
    if (n < this.start.length) {
      return this.start._get(n);
    }

    const k = n - this.start.length;

    if (k >= this.buffer.length || k < 0) {
      throw new tsplus_module_4.IndexOutOfBounds(n, 0, this.length - 1);
    }

    return this.buffer[k];
  }

  _append(a1) {
    const binary = this.binary && isByte(a1);

    if (this.bufferUsed < this.buffer.length && this.chain.compareAndSet(this.bufferUsed, this.bufferUsed + 1)) {
      if (this.binary && !binary) {
        const buffer = new Array(BufferSize);

        for (let i = 0; i < BufferSize; i++) {
          buffer[i] = this.buffer[i];
        }

        buffer[this.bufferUsed] = a1;
        return new AppendN(this.start, buffer, this.bufferUsed + 1, this.chain, this.binary && binary);
      }

      this.buffer[this.bufferUsed] = a1;
      return new AppendN(this.start, this.buffer, this.bufferUsed + 1, this.chain, this.binary && binary);
    } else {
      const buffer = this.binary && binary ? alloc(BufferSize) : new Array(BufferSize);
      buffer[0] = a1;

      const chunk = array_(this.buffer)._take(this.bufferUsed);

      return new AppendN(this.start._concat(chunk), buffer, 1, new tsplus_module_3.AtomicNumber(1), this.binary && binary);
    }
  }

  _copyToArray(n, array) {
    this.start._copyToArray(n, array);

    _copy(this.buffer, 0, array, this.start.length + n, this.bufferUsed);
  }

  [Symbol.iterator]() {
    const k = this._arrayLike();

    return k[Symbol.iterator]();
  }

  _arrayLikeIterator() {
    const array = this._arrayLike();

    let done = false;
    return {
      next: () => {
        if (!done) {
          done = true;
          return {
            value: array,
            done: false
          };
        } else {
          return {
            value: 1,
            done: true
          };
        }
      }
    };
  }

  _reverseArrayLikeIterator() {
    const array = this._arrayLike();

    let done = false;
    return {
      next: () => {
        if (!done) {
          done = true;
          return {
            value: array,
            done: false
          };
        } else {
          return {
            value: 1,
            done: true
          };
        }
      }
    };
  }

}
export const ArrTypeId = /*#__PURE__*/Symbol.for("@effect-ts/core/collection/immutable/Chunk/Arr");
/**
 * Internal Array Chunk
 */

export class Arr extends ChunkInternal {
  constructor() {
    super(...arguments);
    this._typeId = ArrTypeId;
  }

}
/**
 * Internal Plain Array Chunk
 */

export class PlainArr extends Arr {
  constructor(array) {
    super();
    this.array = array;
    this.depth = 0;
    this.left = _Empty;
    this.right = _Empty;
    this.length = array.length;
  }

  get binary() {
    if (typeof this.isBytes !== "undefined") {
      return this.isBytes;
    }

    this.isBytes = this.array.every(isByte);
    return this.isBytes;
  }

  _get(n) {
    if (n >= this.length || n < 0) {
      throw new tsplus_module_4.IndexOutOfBounds(n, 0, this.length - 1);
    }

    return this.array[n];
  }

  _arrayLike() {
    if (!this.binary) {
      return this.array;
    }

    if (this.arrayLikeCache) {
      return this.arrayLikeCache;
    }

    const arr = alloc(this.length);

    this._copyToArray(0, arr);

    this.arrayLikeCache = arr;
    return arr;
  }

  _array() {
    return this.array;
  }

  _materialize() {
    return this;
  }

  _copyToArray(n, array) {
    _copy(this.array, 0, array, n, this.length);
  }

  [Symbol.iterator]() {
    return this.array[Symbol.iterator]();
  }

  _arrayLikeIterator() {
    let done = false;
    return {
      next: () => {
        if (!done) {
          done = true;
          return {
            value: this.array,
            done: false
          };
        } else {
          return {
            value: 1,
            done: true
          };
        }
      }
    };
  }

  _reverseArrayLikeIterator() {
    let done = false;
    return {
      next: () => {
        if (!done) {
          done = true;
          return {
            value: this.array,
            done: false
          };
        } else {
          return {
            value: 1,
            done: true
          };
        }
      }
    };
  }

}
/**
 * Internal Binary Array Chunk
 */

export class Uint8Arr extends Arr {
  constructor(array) {
    super();
    this.array = array;
    this.depth = 0;
    this.left = _Empty;
    this.right = _Empty;
    this.binary = true;
    this.length = array.length;
  }

  _arrayLike() {
    return this.array;
  }

  _get(n) {
    if (n >= this.length || n < 0) {
      throw new tsplus_module_4.IndexOutOfBounds(n, 0, this.length - 1);
    }

    return this.array[n];
  }

  _materialize() {
    return this;
  }

  _copyToArray(n, array) {
    _copy(this.array, 0, array, n, this.length);
  }

  [Symbol.iterator]() {
    return this.array[Symbol.iterator]();
  }

  _arrayLikeIterator() {
    let done = false;
    return {
      next: () => {
        if (!done) {
          done = true;
          return {
            value: this.array,
            done: false
          };
        } else {
          return {
            value: 1,
            done: true
          };
        }
      }
    };
  }

  _reverseArrayLikeIterator() {
    let done = false;
    return {
      next: () => {
        if (!done) {
          done = true;
          return {
            value: this.array,
            done: false
          };
        } else {
          return {
            value: 1,
            done: true
          };
        }
      }
    };
  }

}
export const SliceTypeId = /*#__PURE__*/Symbol.for("@effect-ts/core/collection/immutable/Chunk/Slice");
/**
 * Internal Slice Chunk
 */

export class Slice extends ChunkInternal {
  _get(n) {
    return this.chunk._get(n + this.offset);
  }

  constructor(chunk, offset, length) {
    super();
    this.chunk = chunk;
    this.offset = offset;
    this.length = length;
    this.depth = 0;
    this.left = _Empty;
    this.right = _Empty;
    this._typeId = SliceTypeId;
    this.binary = this.chunk.binary;
  }

  _copyToArray(n, array) {
    let i = 0;
    let j = n;

    while (i < this.length) {
      array[j] = this._get(i);
      i += 1;
      j += 1;
    }
  }

  [Symbol.iterator]() {
    const k = this._arrayLike();

    return k[Symbol.iterator]();
  }

  _arrayLikeIterator() {
    const array = this._arrayLike();

    let done = false;
    return {
      next: () => {
        if (!done) {
          done = true;
          return {
            value: array,
            done: false
          };
        } else {
          return {
            value: 1,
            done: true
          };
        }
      }
    };
  }

  _reverseArrayLikeIterator() {
    const array = this._arrayLike();

    let done = false;
    return {
      next: () => {
        if (!done) {
          done = true;
          return {
            value: array,
            done: false
          };
        } else {
          return {
            value: 1,
            done: true
          };
        }
      }
    };
  }

}
export const SingletonTypeId = /*#__PURE__*/Symbol.for("@effect-ts/core/collection/immutable/Chunk/Singleton");
/**
 * Internal Singleton Chunk
 */

export class Singleton extends ChunkInternal {
  _get(n) {
    if (n === 0) {
      return this.a;
    }

    throw new tsplus_module_4.IndexOutOfBounds(n, 0, this.length - 1);
  }

  constructor(a) {
    super();
    this.a = a;
    this.depth = 0;
    this.left = _Empty;
    this.right = _Empty;
    this.length = 1;
    this._typeId = SingletonTypeId;
    this.binary = isByte(a);
  }

  _copyToArray(n, array) {
    array[n] = this.a;
  }

  [Symbol.iterator]() {
    const k = this._arrayLike();

    return k[Symbol.iterator]();
  }

  _arrayLikeIterator() {
    let done = false;
    return {
      next: () => {
        if (!done) {
          done = true;
          return {
            value: this._arrayLike(),
            done: false
          };
        } else {
          return {
            value: 1,
            done: true
          };
        }
      }
    };
  }

  _reverseArrayLikeIterator() {
    let done = false;
    return {
      next: () => {
        if (!done) {
          done = true;
          return {
            value: this._arrayLike(),
            done: false
          };
        } else {
          return {
            value: 1,
            done: true
          };
        }
      }
    };
  }

}
export const PrependNTypeId = /*#__PURE__*/Symbol.for("@effect-ts/core/collection/immutable/Chunk/PrependN");
/**
 * Internal Prepend Chunk
 */

export class PrependN extends ChunkInternal {
  _get(n) {
    if (n < this.bufferUsed) {
      const k = BufferSize - this.bufferUsed + n;

      if (k >= this.buffer.length || k < 0) {
        throw new tsplus_module_4.IndexOutOfBounds(n, 0, this.length - 1);
      }

      return this.buffer[k];
    }

    return this.end._get(n - this.bufferUsed);
  }

  constructor(end, buffer, bufferUsed, chain, binary) {
    super();
    this.end = end;
    this.buffer = buffer;
    this.bufferUsed = bufferUsed;
    this.chain = chain;
    this.binary = binary;
    this.depth = 0;
    this.left = _Empty;
    this.right = _Empty;
    this._typeId = PrependNTypeId;
    this.length = this.end.length + this.bufferUsed;
  }

  _copyToArray(n, array) {
    const length = Math.min(this.bufferUsed, Math.max(array.length - n, 0));

    _copy(this.buffer, BufferSize - this.bufferUsed, array, n, length);

    this.end._copyToArray(n + length, array);
  }

  prepend(a1) {
    const binary = this.binary && isByte(a1);

    if (this.bufferUsed < this.buffer.length && this.chain.compareAndSet(this.bufferUsed, this.bufferUsed + 1)) {
      if (this.binary && !binary) {
        const buffer = new Array(BufferSize);

        for (let i = 0; i < BufferSize; i++) {
          buffer[i] = this.buffer[i];
        }

        buffer[BufferSize - this.bufferUsed - 1] = a1;
        return new PrependN(this.end, buffer, this.bufferUsed + 1, this.chain, false);
      }

      this.buffer[BufferSize - this.bufferUsed - 1] = a1;
      return new PrependN(this.end, this.buffer, this.bufferUsed + 1, this.chain, this.binary && binary);
    } else {
      const buffer = binary ? alloc(BufferSize) : new Array(BufferSize);
      buffer[BufferSize - 1] = a1;
      const chunk = array_("subarray" in this.buffer ? this.buffer.subarray(this.buffer.length - this.bufferUsed) : this.buffer.slice(this.buffer.length - this.bufferUsed));
      return new PrependN(chunk._concat(this.end), buffer, 1, new tsplus_module_3.AtomicNumber(1), this.binary && binary);
    }
  }

  [Symbol.iterator]() {
    const k = this._arrayLike();

    return k[Symbol.iterator]();
  }

  _arrayLikeIterator() {
    let done = false;
    return {
      next: () => {
        if (!done) {
          done = true;
          return {
            value: this._arrayLike(),
            done: false
          };
        } else {
          return {
            value: 1,
            done: true
          };
        }
      }
    };
  }

  _reverseArrayLikeIterator() {
    let done = false;
    return {
      next: () => {
        if (!done) {
          done = true;
          return {
            value: this._arrayLike(),
            done: false
          };
        } else {
          return {
            value: 1,
            done: true
          };
        }
      }
    };
  }

}
/**
 * Internal copy arrays
 */

export function _copy(src, srcPos, dest, destPos, len) {
  for (let i = srcPos; i < Math.min(src.length, srcPos + len); i++) {
    dest[destPos + i - srcPos] = src[i];
  }

  return dest;
}
export const ConcatTypeId = /*#__PURE__*/Symbol.for("@effect-ts/core/collection/immutable/Chunk/Concat");
/**
 * Internal Concat Chunk
 */

export class Concat extends ChunkInternal {
  _get(n) {
    return n < this.left.length ? this.left._get(n) : this.right._get(n - this.left.length);
  }

  constructor(left, right) {
    super();
    this.left = left;
    this.right = right;
    this._typeId = ConcatTypeId;
    this.depth = 1 + Math.max(this.left.depth, this.right.depth);
    this.length = this.left.length + this.right.length;
    this.binary = this.left.binary && this.right.binary;
  }

  _copyToArray(n, array) {
    this.left._copyToArray(n, array);

    this.right._copyToArray(n + this.left.length, array);
  }

  [Symbol.iterator]() {
    const k = this._arrayLike();

    return k[Symbol.iterator]();
  }

  _arrayLikeIterator() {
    let it = this.left._arrayLikeIterator();

    let i = 0;
    let n = it.next();
    let j = 0;
    return {
      next: () => {
        j++;

        if (i === 0 && n.done) {
          it = this.right._arrayLikeIterator();
          const k = it.next();

          if (k.done) {
            return {
              value: j,
              done: true
            };
          }

          i++;
          n = it.next();
          return k;
        } else {
          if (n.done) {
            return {
              value: j,
              done: true
            };
          }

          const k = n;
          n = it.next();
          return k;
        }
      }
    };
  }

  _reverseArrayLikeIterator() {
    let it = this.right._arrayLikeIterator();

    let i = 0;
    let n = it.next();
    let j = 0;
    return {
      next: () => {
        j++;

        if (i === 0 && n.done) {
          it = this.left._arrayLikeIterator();
          const k = it.next();

          if (k.done) {
            return {
              value: j,
              done: true
            };
          }

          i++;
          n = it.next();
          return k;
        } else {
          if (n.done) {
            return {
              value: j,
              done: true
            };
          }

          const k = n;
          n = it.next();
          return k;
        }
      }
    };
  }

}

function isChunk_1(u) {
  return typeof u === "object" && u != null && ChunkTypeId in u;
}

function array_(array) {
  if (isChunk_1(array)) {
    void 0;
    return array;
  }

  if (array instanceof Uint8Array) {
    return new Uint8Arr(array);
  }

  return new PlainArr(Array.isArray(array) ? array : Array.from(array));
}
/**
 * Builds a chunk from an array.
 *
 * @tsplus static Chunk.Ops from
 */


export function from(array) {
  return array_(array);
}
/**
 * Determines whether this chunk and the specified chunk have the same length
 * and every pair of corresponding elements of this chunk and the specified
 * chunk satisfy the specified predicate.
 *
 * @tsplus fluent Chunk corresponds
 */

function corresponds_1(self, that, f) {
  if (self.length !== that.length) {
    return false;
  }

  const leftIterator = self._arrayLikeIterator();

  const rightIterator = that._arrayLikeIterator();

  let i = 0;
  let j = 0;
  let equal = true;
  let done = false;
  let leftLength = 0;
  let rightLength = 0;
  let left = undefined;
  let right = undefined;
  let leftNext;
  let rightNext;

  while (equal && !done) {
    if (i < leftLength && j < rightLength) {
      if (!f(left[i], right[j])) {
        equal = false;
      }

      i++;
      j++;
    } else if (i === leftLength && (leftNext = leftIterator.next()) && !leftNext.done) {
      left = leftNext.value;
      leftLength = left.length;
      i = 0;
    } else if (j === rightLength && (rightNext = rightIterator.next()) && !rightNext.done) {
      right = rightNext.value;
      rightLength = right.length;
      j = 0;
    } else if (i === leftLength && j === rightLength) {
      done = true;
    } else {
      equal = false;
    }
  }

  return equal;
}
/**
 * Determines whether this chunk and the specified chunk have the same length
 * and every pair of corresponding elements of this chunk and the specified
 * chunk satisfy the specified predicate.
 *
 * @tsplus static Chunk.Aspects corresponds
 */


export const corresponds = (that, f) => self => corresponds_(self, that, f);
//# sourceMappingURL=definition.mjs.map