import * as tsplus_module_1 from "@tsplus/stdlib/collections/Chunk/operations/empty";
import * as tsplus_module_2 from "@tsplus/stdlib/collections/Chunk/operations/append";
import { concreteChunkId } from "@tsplus/stdlib/collections/Chunk/definition";
/**
 * Zips with chunk with the specified chunk to produce a new chunk with
 * pairs of elements from each chunk combined using the specified function
 * `both`. If one chunk is shorter than the other uses the specified
 * function `left` or `right` to map the element that does exist to the
 * result type.
 *
 * @tsplus static Chunk.Aspects zipAllWith
 * @tsplus pipeable Chunk zipAllWith
 */

export function zipAllWith(that, f, left, right) {
  return self => {
    const length = Math.max(self.length, that.length);

    if (length === 0) {
      return tsplus_module_1.empty();
    }

    const leftIterator = self._arrayLikeIterator();

    const rightIterator = that._arrayLikeIterator();

    let i = 0;
    let j = 0;
    let k = 0;
    let leftLength = 0;
    let rightLength = 0;
    let leftArray = undefined;
    let rightArray = undefined;
    let leftNext;
    let rightNext;
    let builder = tsplus_module_1.empty();

    while (i < length) {
      if (j < leftLength && k < rightLength) {
        builder = tsplus_module_2.append(f(leftArray[j], rightArray[k]))(builder);
        i++;
        j++;
        k++;
      } else if (j === leftLength && (leftNext = leftIterator.next()) && !leftNext.done) {
        leftArray = leftNext.value;
        leftLength = leftArray.length;
        j = 0;
      } else if (k === rightLength && (rightNext = rightIterator.next()) && !rightNext.done) {
        rightArray = rightNext.value;
        rightLength = rightArray.length;
        k = 0;
      } else if (j < leftLength) {
        builder = tsplus_module_2.append(left(leftArray[j]))(builder);
        i++;
        j++;
      } else if (k < rightLength) {
        builder = tsplus_module_2.append(right(rightArray[k]))(builder);
        i++;
        k++;
      }
    }

    return builder;
  };
}
//# sourceMappingURL=zipAllWith.mjs.map