import * as tsplus_module_1 from "@tsplus/stdlib/collections/Chunk/operations/empty";
import * as tsplus_module_2 from "@tsplus/stdlib/collections/Chunk/operations/append";
import { concreteChunkId } from "@tsplus/stdlib/collections/Chunk/definition";
/**
 * Zips this chunk with the specified chunk using the specified combiner.
 *
 * @tsplus static Chunk.Aspects zipWith
 * @tsplus pipeable Chunk zipWith
 */

export function zipWith(that, f) {
  return self => {
    const length = Math.min(self.length, that.length);

    if (length === 0) {
      return tsplus_module_1.empty();
    }

    const leftIterator = self._arrayLikeIterator();

    const rightIterator = that._arrayLikeIterator();

    let i = 0;
    let j = 0;
    let k = 0;
    let leftLength = 0;
    let rightLength = 0;
    let left = undefined;
    let right = undefined;
    let leftNext;
    let rightNext;
    let builder = tsplus_module_1.empty();

    while (i < length) {
      if (j < leftLength && k < rightLength) {
        builder = tsplus_module_2.append(f(left[j], right[k]))(builder);
        i++;
        j++;
        k++;
      } else if (j === leftLength && (leftNext = leftIterator.next()) && !leftNext.done) {
        left = leftNext.value;
        leftLength = left.length;
        j = 0;
      } else if (k === rightLength && (rightNext = rightIterator.next()) && !rightNext.done) {
        right = rightNext.value;
        rightLength = right.length;
        k = 0;
      }
    }

    return builder;
  };
}
//# sourceMappingURL=zipWith.mjs.map