import * as tsplus_module_1 from "@tsplus/stdlib/data/Differ/definition";
import * as tsplus_module_2 from "@tsplus/stdlib/collections/List/from";
import * as tsplus_module_3 from "@tsplus/stdlib/collections/List/definition";
import * as tsplus_module_4 from "@tsplus/stdlib/collections/Chunk/operations/concat";
import * as tsplus_module_5 from "@tsplus/stdlib/collections/List/prepend";
import * as tsplus_module_6 from "@tsplus/stdlib/collections/Collection/functions";
import * as tsplus_module_7 from "@tsplus/stdlib/collections/Chunk/definition";
import { chunkPatchInstruction } from "@tsplus/stdlib/collections/Chunk/patch/definition";
/**
 * Applies a chunk patch to a chunk of values to produce a new chunk of
 * values which represents the original chunk of values updated with the
 * changes described by this patch.
 *
 * @tsplus static Differ.Chunk.Patch.Aspects apply
 * @tsplus pipeable Differ.Chunk.Patch apply
 */

export function apply(oldValue, differ) {
  return self => applyLoop(differ, oldValue, tsplus_module_2.make(self));
}
/**
 * @tsplus tailRec
 */

function applyLoop(differ, chunk, patches) {
  var differ_1 = differ,
      chunk_1 = chunk,
      patches_1 = patches;
  var differ_2 = differ,
      chunk_2 = chunk,
      patches_2 = patches;

  while (1) {
    if (tsplus_module_3.isNil(patches_1)) {
      return chunk_1;
    }

    const patch = patches_1.head;
    const nextPatches = patches_1.tail;

    switch (patch._tag) {
      case "Append":
        {
          differ_2 = differ_1;
          chunk_2 = tsplus_module_4.concat(patch.values)(chunk_1);
          patches_2 = nextPatches;
          differ_1 = differ_2;
          chunk_1 = chunk_2;
          patches_1 = patches_2;
          continue;
        }

      case "AndThen":
        {
          differ_2 = differ_1;
          chunk_2 = chunk_1;
          patches_2 = tsplus_module_5.prepend(patch.first)(tsplus_module_5.prepend(patch.second)(nextPatches));
          differ_1 = differ_2;
          chunk_1 = chunk_2;
          patches_1 = patches_2;
          continue;
        }

      case "Empty":
        {
          differ_2 = differ_1;
          chunk_2 = chunk_1;
          patches_2 = nextPatches;
          differ_1 = differ_2;
          chunk_1 = chunk_2;
          patches_1 = patches_2;
          continue;
        }

      case "Slice":
        {
          differ_2 = differ_1;
          chunk_2 = tsplus_module_7.from(tsplus_module_6.toArray(chunk_1).slice(patch.from, patch.until));
          patches_2 = nextPatches;
          differ_1 = differ_2;
          chunk_1 = chunk_2;
          patches_1 = patches_2;
          continue;
        }

      case "Update":
        {
          const array = tsplus_module_6.toArray(chunk_1);
          array[patch.index] = differ_1.patch(patch.patch, array[patch.index]);
          differ_2 = differ_1;
          chunk_2 = tsplus_module_7.from(array);
          patches_2 = nextPatches;
          differ_1 = differ_2;
          chunk_1 = chunk_2;
          patches_1 = patches_2;
          continue;
        }
    }
  }
}
//# sourceMappingURL=apply.mjs.map