import * as tsplus_module_1 from "@tsplus/stdlib/data/Differ/definition";
import * as tsplus_module_2 from "@tsplus/stdlib/collections/Chunk/patch/operations/emptyPatch";
import * as tsplus_module_3 from "@tsplus/stdlib/collections/Chunk/operations/unsafeGet";
import * as tsplus_module_4 from "@tsplus/stdlib/structure/Equals";
import * as tsplus_module_5 from "@tsplus/stdlib/collections/Chunk/patch/operations/combine";
import * as tsplus_module_6 from "@tsplus/stdlib/collections/Chunk/operations/drop";
import { AppendChunkPatch, SliceChunkPatch, UpdateChunkPatch } from "@tsplus/stdlib/collections/Chunk/patch/definition";
/**
 * Constructs a chunk patch from a new and old chunk of values and a differ
 * for the values.
 *
 * @tsplus static Differ.Chunk.Patch.Ops diff
 */

export function diff(oldValue, newValue, differ) {
  let i = 0;
  let patch = tsplus_module_2.emptyPatch();

  while (i < oldValue.length && i < newValue.length) {
    const oldElement = tsplus_module_3.unsafeGet(i)(oldValue);
    const newElement = tsplus_module_3.unsafeGet(i)(newValue);
    const valuePatch = differ.diff(oldElement, newElement);

    if (!tsplus_module_4.equals(valuePatch, differ.empty)) {
      patch = tsplus_module_5.combine(new UpdateChunkPatch(i, valuePatch))(patch);
    }

    i = i + 1;
  }

  if (i < oldValue.length) {
    patch = tsplus_module_5.combine(new SliceChunkPatch(0, i))(patch);
  }

  if (i < newValue.length) {
    patch = tsplus_module_5.combine(new AppendChunkPatch(tsplus_module_6.drop(i)(newValue)))(patch);
  }

  return patch;
}
//# sourceMappingURL=diff.mjs.map