var _a;

import * as tsplus_module_1 from "@tsplus/stdlib/structure/Hash";
import * as tsplus_module_2 from "@tsplus/stdlib/structure/Equals";
import * as tsplus_module_3 from "@tsplus/stdlib/collections/HashMap/definition";
import * as tsplus_module_4 from "@tsplus/stdlib/data/Maybe/definition";
export const getHash_ = getHash_1;
import { fromBitmap, hashFragment, toBitmap } from "@tsplus/stdlib/collections/HashMap/_internal/bitwise";
import { SIZE } from "@tsplus/stdlib/collections/HashMap/_internal/config";
import { isEmptyNode } from "@tsplus/stdlib/collections/HashMap/_internal/node";
import { _K, _V, HashMapSym } from "@tsplus/stdlib/collections/HashMap/definition";
export class HashMapInternal {
  constructor(_editable, _edit, _root, _size) {
    this._editable = _editable;
    this._edit = _edit;
    this._root = _root;
    this._size = _size;
    this[_a] = HashMapSym;
  }

  [(_a = HashMapSym, Symbol.iterator)]() {
    return new HashMapIterator(this, (k, v) => [k, v]);
  }

  [tsplus_module_1.Hash.sym]() {
    let hash = tsplus_module_1.hashString("HashMap");

    for (const item of this) {
      hash ^= tsplus_module_1.combine(tsplus_module_1.hashUnknown(item[0]), tsplus_module_1.hashUnknown(item[1]));
    }

    return tsplus_module_1.optimize(hash);
  }

  [tsplus_module_2.Equals.sym](that) {
    if (tsplus_module_3.isHashMap(that)) {
      void 0;

      if (that._size !== this._size) {
        return false;
      }

      for (const item of this) {
        const elem = getHash_1(that, item[0], tsplus_module_1.hashUnknown(item[0]));

        if (tsplus_module_4.isNone(elem)) {
          return false;
        } else {
          if (!tsplus_module_2.equals(item[1], elem.value)) {
            return false;
          }
        }
      }

      return true;
    }

    return false;
  }

}
export class HashMapIterator {
  constructor(map, f) {
    this.map = map;
    this.f = f;
    this.v = visitLazy(this.map._root, this.f, undefined);
  }

  next() {
    if (tsplus_module_4.isNone(this.v)) {
      return {
        done: true,
        value: undefined
      };
    }

    const v0 = this.v.value;
    this.v = applyCont(v0.cont);
    return {
      done: false,
      value: v0.value
    };
  }

  [Symbol.iterator]() {
    return new HashMapIterator(this.map, this.f);
  }

}
/**
 * @tsplus macro remove
 */

export function realHashMap(_) {//
}
export function applyCont(cont) {
  return cont ? visitLazyChildren(cont[0], cont[1], cont[2], cont[3], cont[4]) : tsplus_module_4.none;
}
export function visitLazy(node, f, cont = undefined) {
  switch (node._tag) {
    case "LeafNode":
      {
        return tsplus_module_4.isSome(node.value) ? tsplus_module_4.some({
          value: f(node.key, node.value.value),
          cont
        }) : applyCont(cont);
      }

    case "CollisionNode":
    case "ArrayNode":
    case "IndexedNode":
      {
        const children = node.children;
        return visitLazyChildren(children.length, children, 0, f, cont);
      }

    default:
      {
        return applyCont(cont);
      }
  }
}
export function visitLazyChildren(len, children, i, f, cont) {
  while (i < len) {
    const child = children[i++];

    if (child && !isEmptyNode(child)) {
      return visitLazy(child, f, [len, children, i, f, cont]);
    }
  }

  return applyCont(cont);
}
/**
 * Lookup the value for the specified key in the `HashMap` using a custom hash.
 *
 * @tsplus fluent HashMap getHash
 */

function getHash_1(self, key, hash) {
  void 0;
  let node = self._root;
  let shift = 0; // eslint-disable-next-line no-constant-condition

  while (true) {
    switch (node._tag) {
      case "LeafNode":
        {
          return tsplus_module_2.equals(key, node.key) ? node.value : tsplus_module_4.none;
        }

      case "CollisionNode":
        {
          if (hash === node.hash) {
            const children = node.children;

            for (let i = 0, len = children.length; i < len; ++i) {
              const child = children[i];
              if ("key" in child && tsplus_module_2.equals(key, child.key)) return child.value;
            }
          }

          return tsplus_module_4.none;
        }

      case "IndexedNode":
        {
          const frag = hashFragment(shift, hash);
          const bit = toBitmap(frag);

          if (node.mask & bit) {
            node = node.children[fromBitmap(node.mask, bit)];
            shift += SIZE;
            break;
          }

          return tsplus_module_4.none;
        }

      case "ArrayNode":
        {
          node = node.children[hashFragment(shift, hash)];

          if (node) {
            shift += SIZE;
            break;
          }

          return tsplus_module_4.none;
        }

      default:
        return tsplus_module_4.none;
    }
  }
}
/**
 * Lookup the value for the specified key in the `HashMap` using a custom hash.
 *
 * @tsplus static HashMap.Aspects getHash
 */


export const getHash = (key, hash) => self => getHash_(self, key, hash);
/**
 * Checks if the specified key has an entry in the `HashMap`.
 *
 * @tsplus fluent HashMap has
 */

export function has_(self, key) {
  return tsplus_module_4.isSome(getHash_1(self, key, tsplus_module_1.hashUnknown(key)));
}
/**
 * Checks if the specified key has an entry in the `HashMap`.
 *
 * @tsplus static HashMap.Aspects has
 */

export const has = key => self => has_(self, key);
export function equals_(self, that) {
  return tsplus_module_2.equals(self, that);
}
export const equals = that => self => equals_(self, that);
//# sourceMappingURL=hashMap.mjs.map