import * as tsplus_module_1 from "@tsplus/stdlib/data/Differ/definition";
import * as tsplus_module_2 from "@tsplus/stdlib/collections/List/from";
import * as tsplus_module_3 from "@tsplus/stdlib/collections/List/definition";
import * as tsplus_module_4 from "@tsplus/stdlib/collections/HashMap/operations/set";
import * as tsplus_module_5 from "@tsplus/stdlib/collections/List/prepend";
import * as tsplus_module_6 from "@tsplus/stdlib/collections/HashMap/operations/remove";
import * as tsplus_module_7 from "@tsplus/stdlib/collections/HashMap/operations/get";
import * as tsplus_module_8 from "@tsplus/stdlib/data/Maybe/fold";
import { hashMapPatchInstruction } from "@tsplus/stdlib/collections/HashMap/patch/definition";
/**
 * Applies a map patch to a map of keys and values to produce a new map of
 * keys and values values which represents the original map of keys and
 * values updated with the changes described by this patch.
 *
 * @tsplus static Differ.HashMap.Patch.Aspects apply
 * @tsplus pipeable Differ.HashMap.Patch apply
 */

export function apply(oldValue, differ) {
  return self => applyLoop(differ, oldValue, tsplus_module_2.make(self));
}
/**
 * @tsplus tailRec
 */

function applyLoop(differ, map, patches) {
  var differ_1 = differ,
      map_1 = map,
      patches_1 = patches;
  var differ_2 = differ,
      map_2 = map,
      patches_2 = patches;

  while (1) {
    if (tsplus_module_3.isNil(patches_1)) {
      return map_1;
    }

    const patch = patches_1.head;
    const nextPatches = patches_1.tail;

    switch (patch._tag) {
      case "Add":
        {
          differ_2 = differ_1;
          map_2 = tsplus_module_4.set(patch.key, patch.value)(map_1);
          patches_2 = nextPatches;
          differ_1 = differ_2;
          map_1 = map_2;
          patches_1 = patches_2;
          continue;
        }

      case "AndThen":
        {
          differ_2 = differ_1;
          map_2 = map_1;
          patches_2 = tsplus_module_5.prepend(patch.first)(tsplus_module_5.prepend(patch.second)(nextPatches));
          differ_1 = differ_2;
          map_1 = map_2;
          patches_1 = patches_2;
          continue;
        }

      case "Empty":
        {
          differ_2 = differ_1;
          map_2 = map_1;
          patches_2 = nextPatches;
          differ_1 = differ_2;
          map_1 = map_2;
          patches_1 = patches_2;
          continue;
        }

      case "Remove":
        {
          differ_2 = differ_1;
          map_2 = tsplus_module_6.remove(patch.key)(map_1);
          patches_2 = nextPatches;
          differ_1 = differ_2;
          map_1 = map_2;
          patches_1 = patches_2;
          continue;
        }

      case "Update":
        {
          differ_2 = differ_1;
          map_2 = tsplus_module_8.fold(() => map_1, oldValue => tsplus_module_4.set(patch.key, differ_1.patch(patch.patch, oldValue))(map_1))(tsplus_module_7.get(patch.key)(map_1));
          patches_2 = nextPatches;
          differ_1 = differ_2;
          map_1 = map_2;
          patches_1 = patches_2;
          continue;
        }
    }
  }
}
//# sourceMappingURL=apply.mjs.map