import * as tsplus_module_1 from "@tsplus/stdlib/data/Differ/definition";
import * as tsplus_module_2 from "@tsplus/stdlib/collections/List/from";
import * as tsplus_module_3 from "@tsplus/stdlib/collections/List/definition";
import * as tsplus_module_4 from "@tsplus/stdlib/collections/HashSet/operations/add";
import * as tsplus_module_5 from "@tsplus/stdlib/collections/List/prepend";
import * as tsplus_module_6 from "@tsplus/stdlib/collections/HashSet/operations/remove";
import { hashSetPatchInstruction } from "@tsplus/stdlib/collections/HashSet/patch/definition";
/**
 * Applies a set patch to a set of values to produce a new set of values
 * which represents the original set of values updated with the changes
 * described by this patch.
 *
 * @tsplus static Differ.HashSet.Patch.Aspects apply
 * @tsplus pipeable Differ.HashSet.Patch apply
 */

export function apply(oldValue) {
  return self => applyLoop(oldValue, tsplus_module_2.make(self));
}
/**
 * @tsplus tailRec
 */

function applyLoop(set, patches) {
  var set_1 = set,
      patches_1 = patches;
  var set_2 = set,
      patches_2 = patches;

  while (1) {
    if (tsplus_module_3.isNil(patches_1)) {
      return set_1;
    }

    const patch = patches_1.head;
    const nextPatches = patches_1.tail;

    switch (patch._tag) {
      case "Add":
        {
          set_2 = tsplus_module_4.add(patch.value)(set_1);
          patches_2 = nextPatches;
          set_1 = set_2;
          patches_1 = patches_2;
          continue;
        }

      case "AndThen":
        {
          set_2 = set_1;
          patches_2 = tsplus_module_5.prepend(patch.first)(tsplus_module_5.prepend(patch.second)(nextPatches));
          set_1 = set_2;
          patches_1 = patches_2;
          continue;
        }

      case "Empty":
        {
          set_2 = set_1;
          patches_2 = nextPatches;
          set_1 = set_2;
          patches_1 = patches_2;
          continue;
        }

      case "Remove":
        {
          set_2 = tsplus_module_6.remove(patch.value)(set_1);
          patches_2 = nextPatches;
          set_1 = set_2;
          patches_1 = patches_2;
          continue;
        }
    }
  }
}
//# sourceMappingURL=apply.mjs.map