import * as tsplus_module_1 from "@tsplus/stdlib/structure/Equals";
import * as tsplus_module_2 from "@tsplus/stdlib/structure/Hash";
/**
 * @tsplus type ImmutableMap
 * @tsplus companion ImmutableMap.Ops
 */

export class ImmutableMap {
  constructor(internalMap) {
    this.internalMap = internalMap;
  }

  [Symbol.iterator]() {
    const iterator = this.internalMap[Symbol.iterator]();
    return {
      next: () => {
        const next = iterator.next();

        if (next.done) {
          return {
            done: true,
            value: undefined
          };
        }

        return {
          done: false,
          value: [next.value[0], next.value[1]]
        };
      }
    };
  }

  [tsplus_module_1.Equals.sym](other) {
    if (other instanceof ImmutableMap && this.internalMap.size === other.internalMap.size) {
      for (const [otherKey, otherValue] of other.internalMap) {
        if (this.internalMap.has(otherKey)) {
          const value = this.internalMap.get(otherKey);

          if (!tsplus_module_1.equals(value, otherValue)) {
            return false;
          }
        } else {
          return false;
        }
      }

      return true;
    }

    return false;
  }

  [tsplus_module_2.Hash.sym]() {
    let hash = tsplus_module_2.hashString("ImmutableMap");

    for (const item of this) {
      hash ^= tsplus_module_2.combine(tsplus_module_2.hashUnknown(item[0]), tsplus_module_2.hashUnknown(item[1]));
    }

    return tsplus_module_2.optimize(hash);
  }

}
/**
 * @tsplus static ImmutableMap.Ops $
 */

export const ImmutableMapAspects = {};
//# sourceMappingURL=definition.mjs.map