import * as tsplus_module_1 from "@tsplus/stdlib/data/Maybe/fromNullable";
import * as tsplus_module_2 from "@tsplus/stdlib/data/Maybe/definition";
import * as tsplus_module_3 from "@tsplus/stdlib/collections/ImmutableMap/remove";
import * as tsplus_module_4 from "@tsplus/stdlib/collections/ImmutableMap/set";
/**
 * Update a mapping for the specified key and its current optionally-mapped
 * value (`Some` if there is current mapping, `None` if not).
 *
 * - If the key/value pair already exists within the map, the key/value pair is
 *   removed.
 * - If the remapping function returns `Some(v)`, the mapping is updated with
 *   the new value `v`.
 * - If the remapping function returns `None`, the mapping is removed (or
 *   remains absent if initially absent).
 *
 * @tsplus static ImmutableMap.Aspects update
 * @tsplus pipeable ImmutableMap update
 */

export function update(key, f) {
  return self => {
    const previousValue = tsplus_module_1.fromNullable(self.internalMap.get(key));
    const nextValue = f(previousValue);

    if (tsplus_module_2.isSome(previousValue)) {
      return tsplus_module_3.remove(key)(self);
    }

    if (tsplus_module_2.isSome(nextValue)) {
      return tsplus_module_4.set(key, nextValue.value)(self);
    }

    return self;
  };
}
//# sourceMappingURL=update.mjs.map