var _a, _b;

import * as tsplus_module_1 from "@tsplus/stdlib/structure/Hash";
import * as tsplus_module_2 from "@tsplus/stdlib/structure/Equals";
export const isNil = isNil_1;
export const length = length_1;
export const equalsWith = equalsWith_1;
/*
 * This file is ported from
 *
 * Scala (https://www.scala-lang.org)
 *
 * Copyright EPFL and Lightbend, Inc.
 *
 * Licensed under Apache License 2.0
 * (http://www.apache.org/licenses/LICENSE-2.0).
 */

export const ListTypeId = /*#__PURE__*/Symbol.for("@tsplus/collections/List");
export const _A = /*#__PURE__*/Symbol.for("@tsplus/collections/List/_A");
export const List = {
  $: {}
};
/**
 * @tsplus type List/Cons
 */

export class Cons {
  constructor(head, tail) {
    this.head = head;
    this.tail = tail;
    this._tag = "Cons";
    this[_a] = ListTypeId;
  }

  [(_a = ListTypeId, Symbol.iterator)]() {
    let done = false; // eslint-disable-next-line @typescript-eslint/no-this-alias

    let these = this;
    return {
      next() {
        if (done) {
          return this.return();
        }

        if (these._tag === "Nil") {
          done = true;
          return this.return();
        }

        const value = these.head;
        these = these.tail;
        return {
          done,
          value
        };
      },

      return(value) {
        if (!done) {
          done = true;
        }

        return {
          done: true,
          value
        };
      }

    };
  }

  [tsplus_module_1.Hash.sym]() {
    return tsplus_module_1.hashIterator(this[Symbol.iterator]());
  }

  [tsplus_module_2.Equals.sym](that) {
    return that instanceof Cons && equalsWith_1(that, tsplus_module_2.equals)(this);
  }

}
/**
 * @tsplus type List/Nil
 */

export class Nil {
  constructor() {
    this._tag = "Nil";
    this[_b] = ListTypeId;
  }

  [(_b = ListTypeId, Symbol.iterator)]() {
    return {
      next() {
        return {
          done: true,
          value: undefined
        };
      }

    };
  }

  [tsplus_module_1.Hash.sym]() {
    return tsplus_module_1.hashIterator(this[Symbol.iterator]());
  }

  [tsplus_module_2.Equals.sym](that) {
    return that instanceof Nil;
  }

}
export const _Nil = /*#__PURE__*/new Nil();
/**
 * @tsplus unify List
 * @tsplus unify List/Nil
 * @tsplus unify List/Cons
 */

export function unify(self) {
  return self;
}
/**
 * @tsplus static List.Ops nil
 */

export function nil() {
  return _Nil;
}
/**
 * @tsplus static List.Ops cons
 */

export function cons(head, tail) {
  return new Cons(head, tail);
}
/**
 * @tsplus fluent List isNil
 */

function isNil_1(self) {
  return self._tag === "Nil";
}
/**
 * @tsplus fluent List isCons
 */


export function isCons(self) {
  return self._tag === "Cons";
}
/**
 * Returns the number of elements contained in a `List`
 *
 * @tsplus getter List length
 */

function length_1(self) {
  let these = self;
  let len = 0;

  while (!isNil_1(these)) {
    len += 1;
    these = these.tail;
  }

  return len;
}
/**
 * @tsplus static List.Aspects equalsWith
 * @tsplus pipeable List equalsWith
 */


function equalsWith_1(that, f) {
  return self => {
    if (self === that) {
      return true;
    } else if (length_1(self) !== length_1(that)) {
      return false;
    } else {
      const i0 = self[Symbol.iterator]();
      const i1 = that[Symbol.iterator]();
      let a;
      let b;

      while (!(a = i0.next()).done && !(b = i1.next()).done) {
        if (!f(a.value, b.value)) {
          return false;
        }
      }

      return true;
    }
  };
}

export function equals(that) {
  return self => equalsWith_1(that, tsplus_module_2.equals)(self);
}
export function isList(u) {
  return typeof u === "object" && u != null && ListTypeId in u;
}
//# sourceMappingURL=definition.mjs.map