import * as tsplus_module_1 from "@tsplus/stdlib/data/Maybe/fold";
import * as tsplus_module_2 from "@tsplus/stdlib/data/Maybe/definition";
export const RedBlackTreeSym = /*#__PURE__*/Symbol.for("@tsplus/stdlib/collections/RedBlackTree");
export const _K = /*#__PURE__*/Symbol.for("@tsplus/stdlib/collections/RedBlackTree/K");
export const _V = /*#__PURE__*/Symbol.for("@tsplus/stdlib/collections/RedBlackTree/V");
export const RedBlackTree = {
  $: {}
};
export class RedBlackTreeIterator {
  constructor(self, stack, direction) {
    this.self = self;
    this.stack = stack;
    this.direction = direction;
    this.count = 0;
  }
  /**
   * Clones the iterator
   */


  clone() {
    return new RedBlackTreeIterator(this.self, this.stack.slice(), this.direction);
  }
  /**
   * Reverse the traversal direction
   */


  reversed() {
    return new RedBlackTreeIterator(this.self, this.stack.slice(), this.direction === "Forward" ? "Backward" : "Forward");
  }
  /**
   * Iterator next
   */


  next() {
    const entry = this.entry;
    this.count++;

    if (this.direction === "Forward") {
      this.moveNext();
    } else {
      this.movePrev();
    }

    return tsplus_module_1.fold(() => ({
      done: true,
      value: this.count
    }), kv => ({
      done: false,
      value: kv
    }))(entry);
  }
  /**
   * Returns the key
   */


  get key() {
    if (this.stack.length > 0) {
      return tsplus_module_2.some(this.stack[this.stack.length - 1].key);
    }

    return tsplus_module_2.none;
  }
  /**
   * Returns the value
   */


  get value() {
    if (this.stack.length > 0) {
      return tsplus_module_2.some(this.stack[this.stack.length - 1].value);
    }

    return tsplus_module_2.none;
  }
  /**
   * Returns the key
   */


  get entry() {
    if (this.stack.length > 0) {
      return tsplus_module_2.some([this.stack[this.stack.length - 1].key, this.stack[this.stack.length - 1].value]);
    }

    return tsplus_module_2.none;
  }
  /**
   * Returns the position of this iterator in the sorted list
   */


  get index() {
    let idx = 0;
    const stack = this.stack;

    if (stack.length === 0) {
      const r = this.self.root;

      if (r) {
        return r.count;
      }

      return 0;
    } else if (stack[stack.length - 1].left) {
      idx = stack[stack.length - 1].left.count;
    }

    for (let s = stack.length - 2; s >= 0; --s) {
      if (stack[s + 1] === stack[s].right) {
        ;
        ++idx;

        if (stack[s].left) {
          idx += stack[s].left.count;
        }
      }
    }

    return idx;
  }
  /**
   * Advances iterator to next element in list
   */


  moveNext() {
    const stack = this.stack;

    if (stack.length === 0) {
      return;
    }

    let n = stack[stack.length - 1];

    if (n.right) {
      n = n.right;

      while (n) {
        stack.push(n);
        n = n.left;
      }
    } else {
      stack.pop();

      while (stack.length > 0 && stack[stack.length - 1].right === n) {
        n = stack[stack.length - 1];
        stack.pop();
      }
    }
  }
  /**
   * Checks if there is a next element
   */


  get hasNext() {
    const stack = this.stack;

    if (stack.length === 0) {
      return false;
    }

    if (stack[stack.length - 1].right) {
      return true;
    }

    for (let s = stack.length - 1; s > 0; --s) {
      if (stack[s - 1].left === stack[s]) {
        return true;
      }
    }

    return false;
  }
  /**
   * Advances iterator to previous element in list
   */


  movePrev() {
    const stack = this.stack;

    if (stack.length === 0) {
      return;
    }

    let n = stack[stack.length - 1];

    if (n && n.left) {
      n = n.left;

      while (n) {
        stack.push(n);
        n = n.right;
      }
    } else {
      stack.pop();

      while (stack.length > 0 && stack[stack.length - 1].left === n) {
        n = stack[stack.length - 1];
        stack.pop();
      }
    }
  }
  /**
   * Checks if there is a previous element
   */


  get hasPrev() {
    const stack = this.stack;

    if (stack.length === 0) {
      return false;
    }

    if (stack[stack.length - 1].left) {
      return true;
    }

    for (let s = stack.length - 1; s > 0; --s) {
      if (stack[s - 1].right === stack[s]) {
        return true;
      }
    }

    return false;
  }

}
//# sourceMappingURL=definition.mjs.map