/**
 * @tsplus type RedBlackTree.Node
 * @tsplus companion RedBlackTree.Node.Ops
 */
export class Node {
  constructor(color, key, value, left, right, count) {
    this.color = color;
    this.key = key;
    this.value = value;
    this.left = left;
    this.right = right;
    this.count = count;
  }

}
/**
 * @tsplus fluent RedBlackTree.Node clone
 */

export function clone(node) {
  return new Node(node.color, node.key, node.value, node.left, node.right, node.count);
}
/**
 * @tsplus fluent RedBlackTree.Node swap
 */

export function swap(n, v) {
  n.key = v.key;
  n.value = v.value;
  n.left = v.left;
  n.right = v.right;
  n.color = v.color;
  n.count = v.count;
}
/**
 * @tsplus fluent RedBlackTree.Node repaint
 */

export function repaint(node, color) {
  return new Node(color, node.key, node.value, node.left, node.right, node.count);
}
/**
 * @tsplus fluent RedBlackTree.Node recount
 */

export function recount(node) {
  node.count = 1 + (node.left?.count ?? 0) + (node.right?.count ?? 0);
}
//# sourceMappingURL=node.mjs.map