import * as tsplus_module_1 from "@tsplus/stdlib/collections/List/definition";
import * as tsplus_module_2 from "@tsplus/stdlib/structure/Hash";
import * as tsplus_module_3 from "@tsplus/stdlib/structure/Equals";
import * as tsplus_module_4 from "@tsplus/stdlib/exceptions";
import * as tsplus_module_5 from "@tsplus/stdlib/collections/List/reduce";
import * as tsplus_module_6 from "@tsplus/stdlib/collections/List/unsafeTail";
/*
 * This file is ported from
 *
 * Scala (https://www.scala-lang.org)
 *
 * Copyright EPFL and Lightbend, Inc.
 *
 * Licensed under Apache License 2.0
 * (http://www.apache.org/licenses/LICENSE-2.0).
 */

/**
 * @tsplus type ListBuffer
 * @tsplus companion ListBuffer.Ops
 */

export class ListBuffer {
  constructor() {
    this.first = tsplus_module_1.nil();
    this.last0 = undefined;
    this.len = 0;
  }

  [Symbol.iterator]() {
    return this.first[Symbol.iterator]();
  }

  [tsplus_module_2.Hash.sym]() {
    return tsplus_module_2.hashRandomCached(this);
  }

  [tsplus_module_3.Equals.sym](that) {
    return this === that;
  }

  static empty() {
    return new ListBuffer();
  }

  static from(as) {
    const buf = new ListBuffer();

    for (const a of as) {
      buf.append(a);
    }

    return buf;
  }

  get length() {
    return this.len;
  }

  get isEmpty() {
    return this.len === 0;
  }

  get unsafeHead() {
    if (this.isEmpty) {
      return undefined;
    }

    return this.first.head;
  }

  get unsafeTail() {
    if (this.isEmpty) {
      return undefined;
    }

    return this.first.tail;
  }

  append(elem) {
    const last1 = tsplus_module_1.cons(elem, tsplus_module_1.nil());

    if (this.len === 0) {
      this.first = last1;
    } else {
      this.last0.tail = last1;
    }

    this.last0 = last1;
    this.len += 1;
    return this;
  }

  prepend(elem) {
    this.insert(0, elem);
    return this;
  }

  unprepend() {
    if (this.isEmpty) {
      throw new tsplus_module_4.NoSuchElement();
    }

    const h = this.first.head;
    this.first = this.first.tail;
    this.len -= 1;
    return h;
  }

  get toList() {
    return this.first;
  }

  insert(idx, elem) {
    if (idx < 0 || idx > this.len) {
      throw new tsplus_module_4.IndexOutOfBounds(idx, 0, this.len - 1);
    }

    if (idx === this.len) {
      this.append(elem);
    } else {
      const p = this.locate(idx);
      const nx = tsplus_module_1.cons(elem, this.getNext(p));

      if (p === undefined) {
        this.first = nx;
      } else {
        ;
        p.tail = nx;
      }

      this.len += 1;
    }

    return this;
  }

  reduce(b, f) {
    return tsplus_module_5.reduce(b, f)(this.first);
  }

  getNext(p) {
    if (p === undefined) {
      return this.first;
    } else {
      return tsplus_module_6.unsafeTail(p);
    }
  }

  locate(i) {
    if (i === 0) {
      return undefined;
    } else if (i === this.len) {
      return this.last0;
    } else {
      let p = this.first;

      for (let j = i - 1; j > 0; j--) {
        p = tsplus_module_6.unsafeTail(p);
      }

      return p;
    }
  }

}
/**
 * @tsplus getter ListBuffer toCollection
 */

export function toCollection(self) {
  return self;
}
//# sourceMappingURL=ListBuffer.mjs.map