import * as tsplus_module_1 from "@tsplus/stdlib/data/Maybe/definition";
export const IterableWeakMap = {
  $: {}
};
/**
 * @tsplus pipeable-index IterableWeakMap
 * @tsplus static IterableWeakMap.Aspects getMaybe
 * @tsplus pipeable IterableWeakMap getMaybe
 */

export function getMaybe(key) {
  return self => self.has(key) ? tsplus_module_1.some(self.get(key)) : tsplus_module_1.none;
}
/**
 * @tsplus static IterableWeakMap.Ops make
 * @tsplus static IterableWeakMap.Ops __call
 */

export function make(iterable) {
  return new ConcreteImpl(iterable);
}

class WeakImpl {
  static cleanup({
    ref,
    set
  }) {
    set.delete(ref);
  }

  constructor(iterable) {
    this.weakMap = new WeakMap();
    this.refSet = new Set();
    this.finalizationGroup = new FinalizationRegistry(WeakImpl.cleanup);

    for (const [key, value] of iterable) {
      this.set(key, value);
    }
  }

  set(key, value) {
    const ref = new WeakRef(key);
    this.weakMap.set(key, {
      value,
      ref
    });
    this.refSet.add(ref);
    this.finalizationGroup.register(key, {
      set: this.refSet,
      ref
    }, ref);
    return this;
  }

  get(key) {
    const entry = this.weakMap.get(key);
    return entry && entry.value;
  }

  delete(key) {
    const entry = this.weakMap.get(key);

    if (!entry) {
      return false;
    }

    this.weakMap.delete(key);
    this.refSet.delete(entry.ref);
    this.finalizationGroup.unregister(entry.ref);
    return true;
  }

  [Symbol.iterator]() {
    return this.entries();
  }

  *iterator() {
    for (const ref of this.refSet) {
      const key = ref.deref();
      if (!key) continue;
      const {
        value
      } = this.weakMap.get(key);
      yield [key, value];
    }
  }

  *entries() {
    for (const ref of this.refSet) {
      const key = ref.deref();
      if (!key) continue;
      const {
        value
      } = this.weakMap.get(key);
      yield [key, value];
    }
  }

  *keys() {
    for (const [key] of this) {
      yield key;
    }
  }

  *values() {
    for (const [, value] of this) {
      yield value;
    }
  }

  has(key) {
    return this.weakMap.has(key);
  }

}

const ConcreteImpl = typeof FinalizationRegistry !== "undefined" ? WeakImpl : Map;
/**
 * @tsplus getter IterableWeakMap toCollection
 */

export function toCollection(self) {
  return self;
}
//# sourceMappingURL=IterableWeakMap.mjs.map