import * as tsplus_module_1 from "@tsplus/stdlib/data/Differ/definition";
import * as tsplus_module_2 from "@tsplus/stdlib/collections/List/from";
import * as tsplus_module_3 from "@tsplus/stdlib/collections/List/definition";
import * as tsplus_module_4 from "@tsplus/stdlib/collections/List/prepend";
import * as tsplus_module_5 from "@tsplus/stdlib/data/Either/definition";
import { orPatchInstruction } from "@tsplus/stdlib/data/Differ/OrPatch/definition";
/**
 * Applies an or patch to a value to produce a new value which represents
 * the original value updated with the changes described by this patch.
 *
 * @tsplus static Differ.Or.Patch.Aspects apply
 * @tsplus pipeable Differ.Or.Patch apply
 */

export function apply(oldValue, left, right) {
  return self => applyLoop(left, right, oldValue, tsplus_module_2.make(self));
}
/**
 * @tsplus tailRec
 */

function applyLoop(left, right, either, patches) {
  var left_1 = left,
      right_1 = right,
      either_1 = either,
      patches_1 = patches;
  var left_2 = left,
      right_2 = right,
      either_2 = either,
      patches_2 = patches;

  while (1) {
    if (tsplus_module_3.isNil(patches_1)) {
      return either_1;
    }

    const patch = patches_1.head;
    const nextPatches = patches_1.tail;

    switch (patch._tag) {
      case "AndThen":
        {
          left_2 = left_1;
          right_2 = right_1;
          either_2 = either_1;
          patches_2 = tsplus_module_4.prepend(patch.first)(tsplus_module_4.prepend(patch.second)(nextPatches));
          left_1 = left_2;
          right_1 = right_2;
          either_1 = either_2;
          patches_1 = patches_2;
          continue;
        }

      case "Empty":
        {
          left_2 = left_1;
          right_2 = right_1;
          either_2 = either_1;
          patches_2 = nextPatches;
          left_1 = left_2;
          right_1 = right_2;
          either_1 = either_2;
          patches_1 = patches_2;
          continue;
        }

      case "UpdateLeft":
        {
          switch (either_1._tag) {
            case "Left":
              {
                left_2 = left_1;
                right_2 = right_1;
                either_2 = tsplus_module_5.left(left_1.patch(patch.patch, either_1.left));
                patches_2 = nextPatches;
                left_1 = left_2;
                right_1 = right_2;
                either_1 = either_2;
                patches_1 = patches_2;
                continue;
              }

            case "Right":
              {
                left_2 = left_1;
                right_2 = right_1;
                either_2 = either_1;
                patches_2 = nextPatches;
                left_1 = left_2;
                right_1 = right_2;
                either_1 = either_2;
                patches_1 = patches_2;
                continue;
              }
          }
        }

      case "UpdateRight":
        {
          switch (either_1._tag) {
            case "Left":
              {
                left_2 = left_1;
                right_2 = right_1;
                either_2 = either_1;
                patches_2 = nextPatches;
                left_1 = left_2;
                right_1 = right_2;
                either_1 = either_2;
                patches_1 = patches_2;
                continue;
              }

            case "Right":
              {
                left_2 = left_1;
                right_2 = right_1;
                either_2 = tsplus_module_5.right(right_1.patch(patch.patch, either_1.right));
                patches_2 = nextPatches;
                left_1 = left_2;
                right_1 = right_2;
                either_1 = either_2;
                patches_1 = patches_2;
                continue;
              }
          }
        }

      case "SetLeft":
        {
          left_2 = left_1;
          right_2 = right_1;
          either_2 = tsplus_module_5.left(patch.value);
          patches_2 = nextPatches;
          left_1 = left_2;
          right_1 = right_2;
          either_1 = either_2;
          patches_1 = patches_2;
          continue;
        }

      case "SetRight":
        {
          left_2 = left_1;
          right_2 = right_1;
          either_2 = tsplus_module_5.right(patch.value);
          patches_2 = nextPatches;
          left_1 = left_2;
          right_1 = right_2;
          either_1 = either_2;
          patches_1 = patches_2;
          continue;
        }
    }
  }
}
//# sourceMappingURL=apply.mjs.map