import * as tsplus_module_1 from "@tsplus/stdlib/data/Differ/definition";
import * as tsplus_module_2 from "@tsplus/stdlib/structure/Equals";
import { EmptyOrPatch, SetLeftOrPatch, SetRightOrPatch, UpdateLeftOrPatch, UpdateRightOrPatch } from "@tsplus/stdlib/data/Differ/OrPatch/definition";
/**
 * Constructs an `OrPatch` from a new and old value and a differ for the
 * values.
 *
 * @tsplus static Differ.Or.Patch.Ops diff
 */

export function diff(oldValue, newValue, left, right) {
  switch (oldValue._tag) {
    case "Left":
      {
        switch (newValue._tag) {
          case "Left":
            {
              const valuePatch = left.diff(oldValue.left, newValue.left);

              if (tsplus_module_2.equals(valuePatch, left.empty)) {
                return new EmptyOrPatch();
              }

              return new UpdateLeftOrPatch(valuePatch);
            }

          case "Right":
            {
              return new SetRightOrPatch(newValue.right);
            }
        }
      }

    case "Right":
      {
        switch (newValue._tag) {
          case "Left":
            {
              return new SetLeftOrPatch(newValue.left);
            }

          case "Right":
            {
              const valuePatch = right.diff(oldValue.right, newValue.right);

              if (tsplus_module_2.equals(valuePatch, right.empty)) {
                return new EmptyOrPatch();
              }

              return new UpdateRightOrPatch(valuePatch);
            }
        }
      }
  }
}
//# sourceMappingURL=diff.mjs.map