import * as tsplus_module_1 from "@tsplus/stdlib/data/Function";
import * as tsplus_module_2 from "@tsplus/stdlib/structure/Equals";
import * as tsplus_module_3 from "@tsplus/stdlib/data/Differ/operations/make";
import { constant } from "@tsplus/stdlib/data/Function";
/**
 * A variant of `update` that allows specifying the function that will be used
 * to combine old values with new values.
 *
 * @tsplus static Differ.Ops updateWith
 */

export function updateWith(f) {
  return tsplus_module_3.make({
    empty: tsplus_module_1.identity,
    combine: (first, second) => {
      if (first === tsplus_module_1.identity) {
        return second;
      }

      if (second === tsplus_module_1.identity) {
        return first;
      }

      return a => second(first(a));
    },
    diff: (oldValue, newValue) => {
      if (tsplus_module_2.equals(oldValue, newValue)) {
        return tsplus_module_1.identity;
      }

      return constant(newValue);
    },
    patch: (patch, oldValue) => f(oldValue, patch(oldValue))
  });
}
//# sourceMappingURL=updateWith.mjs.map