import * as tsplus_module_1 from "@tsplus/stdlib/data/Maybe/definition";
import * as tsplus_module_2 from "@tsplus/stdlib/structure/Equals";
export const Either = {
  $: {}
};
/**
 * @tsplus unify Either
 * @tsplus unify Either.Left
 * @tsplus unify Either.Right
 */

export function unifyEither(self) {
  return self;
}
/**
 * Returns `true` if the `Either` is an instance of `Left`, `false` otherwise.
 *
 * @tsplus fluent Either isLeft
 */

export function isLeft(ma) {
  return ma._tag === "Left";
}
/**
 * Returns `true` if the `Either` is an instance of `Right`, `false` otherwise.
 *
 * @tsplus fluent Either isRight
 */

export function isRight(ma) {
  return ma._tag === "Right";
}
/**
 * @tsplus getter Either left
 */

export function getLeft(self) {
  return self._tag === "Left" ? tsplus_module_1.some(self.left) : tsplus_module_1.none;
}
/**
 * @tsplus static Either.Ops isEither
 */

export function isEither(u) {
  return typeof u === "object" && u != null && "_tag" in u && (u["_tag"] === "Left" || u["_tag"] === "Right");
}
/**
 * @tsplus getter Either right
 */

export function getRight(self) {
  return self._tag === "Right" ? tsplus_module_1.some(self.right) : tsplus_module_1.none;
}
/**
 * Constructs a new `Either` holding a `Right` value. This usually represents a
 * successful value due to the right bias of this structure.
 *
 * @tsplus static Either.Ops right
 * @tsplus static Either.Ops __call
 */

export function right(a) {
  return {
    _tag: "Right",
    right: a
  };
}
/**
 * Constructs a new `Either` holding a `Right` value. This usually represents a
 * successful value due to the right bias of this structure.
 *
 * @tsplus static Either.Ops rightW
 */

export function rightW(a) {
  return {
    _tag: "Right",
    right: a
  };
}
/**
 * Constructs a new `Either` holding a `Left` value. This usually represents a
 * failure, due to the right-bias of this structure.
 *
 * @tsplus static Either.Ops left
 */

export function left(e) {
  return {
    _tag: "Left",
    left: e
  };
}
/**
 * Constructs a new `Either` holding a `Left` value. This usually represents a
 * failure, due to the right-bias of this structure.
 *
 * @tsplus static Either.Ops leftW
 */

export function leftW(e) {
  return {
    _tag: "Left",
    left: e
  };
}
/**
 * Widen left side `Either[E, A] => Either[E | E1, A]`.
 *
 * @tsplus getter Either widenE
 * @tsplus macro identity
 */

export function widenE_(self) {
  return self;
}
/**
 * Widen left side `Either[E, A] => Either[E | E1, A]`.
 *
 * @tsplus static Either.Aspects widenE
 */

export function widenE() {
  return (
    /**
     * @tsplus macro identity
     */
    self => self
  );
}
/**
 * Widen left side `Either[E, A] => Either[E | E1, A]`.
 *
 * @tsplus getter Either widenA
 * @tsplus macro identity
 */

export function widenA_(self) {
  return self;
}
/**
 * Widen right side `Either[E, A] => Either[E, A | A1]`.
 *
 * @tsplus static Either.Aspects widenA
 */

export function widenA() {
  return (
    /**
     * @tsplus macro identity
     */
    self => self
  );
}
/**
 * @tsplus operator Either ==
 */

export function equals(a, b) {
  return tsplus_module_2.equals(a, b);
}
//# sourceMappingURL=definition.mjs.map