import * as tsplus_module_1 from "@tsplus/stdlib/prelude/DSL/flatMapF";
import * as tsplus_module_2 from "@tsplus/stdlib/prelude/DSL/succeedF";
import * as tsplus_module_3 from "@tsplus/stdlib/collections/List/reverse";
import * as tsplus_module_4 from "@tsplus/stdlib/exceptions";
import * as tsplus_module_5 from "@tsplus/stdlib/collections/List/prepend";
import * as tsplus_module_6 from "@tsplus/stdlib/collections/List/empty";
export class GenHKT {
  constructor(effect) {
    this.effect = effect;
  }

  *[Symbol.iterator]() {
    return yield this;
  }

}
export class GenLazyHKT {
  constructor(effect) {
    this.effect = effect;
  }

  *[Symbol.iterator]() {
    return yield this;
  }

}

const adapter = _ => {
  return new GenHKT(_);
};

const adapterLazy = _ => {
  return new GenHKT(_);
};
/**
 * To be used with multi-shot monads, required adapter to be lazy and is O(n^2)
 * perf wise because the generator needs to be replayed.
 *
 * @tsplus static DSL genWithHistoryF
 */


export function genWithHistoryF(F, config) {
  return f => {
    const flatMap_ = tsplus_module_1.flatMapF_(F);
    const succeed = tsplus_module_2.succeedF(F);
    return flatMap_(succeed({}), () => {
      function run(replayStack) {
        const iterator = f(config?.adapter ? config.adapter : adapterLazy);
        let state = iterator.next();

        for (const a of tsplus_module_3.reverse(replayStack)) {
          if (state.done) {
            throw new tsplus_module_4.PrematureGeneratorExit();
          }

          state = iterator.next(a);
        }

        if (state.done) {
          return succeed(state.value);
        }

        return flatMap_(state.value["effect"](), val => {
          return run(tsplus_module_5.prepend(val)(replayStack));
        });
      }

      return run(tsplus_module_6.empty());
    });
  };
}
/**
 * To be used in one-shot monads, adapter is eager and perf is native.
 *
 * @tsplus static DSL genF
 */

export function genF(F, config) {
  return f => {
    const flatMap_ = tsplus_module_1.flatMapF_(F);
    const succeed = tsplus_module_2.succeedF(F);
    return flatMap_(succeed({}), () => {
      const iterator = f(config?.adapter ? config.adapter : adapter);
      const state = iterator.next();

      function run(state) {
        if (state.done) {
          return succeed(state.value);
        }

        return flatMap_(state.value["effect"], val => {
          const next = iterator.next(val);
          return run(next);
        });
      }

      return run(state);
    });
  };
}
//# sourceMappingURL=genF.mjs.map