import * as tsplus_module_1 from "@tsplus/stdlib/prelude/Ord/definition";
import * as tsplus_module_2 from "@tsplus/stdlib/prelude/Equivalence/definition";
export const gt = gt_1;
export const lt = lt_1;
export const max = max_1;
export const min = min_1;
/**
 * Contramap Ord input
 *
 * @tsplus fluent Ord contramap
 */

export function contramap(fa, f) {
  return tsplus_module_1.Ord((x, y) => fa.compare(f(x), f(y)));
}
/**
 * Test whether one value is _strictly greater than_ another
 *
 * @tsplus getter Ord gt
 */

function gt_1(O) {
  return (x, y) => O.compare(x, y) === 1;
}
/**
 * Test whether one value is _non-strictly less than_ another
 *
 * @tsplus getter Ord leq
 */


export function leq(O) {
  return (x, y) => O.compare(x, y) !== 1;
}
/**
 * Test whether one value is _strictly less than_ another
 *
 * @tsplus getter Ord lt
 */

function lt_1(O) {
  return (x, y) => O.compare(x, y) === -1;
}
/**
 * Take the maximum of two values. If they are considered equal, the first argument is chosen
 *
 * @tsplus getter Ord max
 */


function max_1(O) {
  return (x, y) => O.compare(x, y) === -1 ? y : x;
}
/**
 * Take the minimum of two values. If they are considered equal, the first argument is chosen
 *
 * @tsplus getter Ord min
 */


function min_1(O) {
  return (x, y) => O.compare(x, y) === 1 ? y : x;
}
/**
 * Test whether a value is between a minimum and a maximum (inclusive)
 *
 * @tsplus getter Ord between
 */


export function between(O) {
  const lessThanO = lt_1(O);
  const greaterThanO = gt_1(O);
  return (low, hi) => x => lessThanO(x, low) || greaterThanO(x, hi) ? false : true;
}
/**
 * Clamp a value between a minimum and a maximum
 *
 * @tsplus getter Ord clamp
 */

export function clamp(O) {
  const minO = min_1(O);
  const maxO = max_1(O);
  return (low, hi) => x => maxO(minO(x, hi), low);
}
/**
 * Get the dual of an Ord
 *
 * @tsplus getter Ord inverted
 */

export function inverted(O) {
  return tsplus_module_1.Ord((x, y) => O.compare(y, x));
}
/**
 * Get an instance of Equal
 *
 * @tsplus getter Ord equivalence
 */

export function getEquivalence(O) {
  return tsplus_module_2.Equivalence((x, y) => O.compare(x, y) === 0);
}
//# sourceMappingURL=operations.mjs.map