/**
 * Use a `Covariant<F>` and a `Recursive.Fn` function to perform a *depth-first* reduction
 * of the `Recursive<F>` structure.  The supplied function will receive the current term
 * with all of its recursive elements replaced by the computed value of its children,
 * i.e. a _catamorphism_
 *
 * @tsplus static Recursive/Aspects fold
 * @tsplus pipeable Recursive fold
 */
export function fold(F, f) {
  return self => go(self);

  function go(term) {
    return f(F.map(go)(term.caseValue));
  }
}
//# sourceMappingURL=fold.mjs.map