import * as tsplus_module_1 from "@tsplus/stdlib/prelude/Recursive/Annotated";
import { Annotated } from "@tsplus/stdlib/prelude/Recursive/Annotated";
/**
 * Use a `Covariant<F>` and a `Annotated.Fn` function to perform a *depth-first* reduction
 * of the `Recursive<F>` structure. The supplied function will receive the current term
 * with all of its recursive elements replaced with *both* the computed value of the
 * sub-structure *and* the value of the computation for all of _its_ children.
 * i.e. _histomorphism_
 *
 * @tsplus static Recursive/Aspects foldAnnotated
 * @tsplus pipeable Recursive foldAnnotated
 */

export function foldAnnotated(F, f) {
  return self => foldAnnotated_(self, F, f);
}

function foldAnnotated_(self, F, f) {
  function annotate(recursive) {
    const calc = F.map(annotate)(recursive.caseValue);
    return tsplus_module_1.makeAnnotated(calc, f(calc));
  }

  return annotate(self).annotations;
}
//# sourceMappingURL=foldAnnotated.mjs.map