"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.forAny = forAny;

var _definition = /*#__PURE__*/require("@tsplus/stdlib/collections/Chunk/definition");

/**
 * Determines whether a predicate is satisfied for all elements of this chunk.
 *
 * @tsplus static Chunk.Aspects forAny
 * @tsplus pipeable Chunk forAny
 */
function forAny(f) {
  return self => {
    const iterator = self._arrayLikeIterator();

    let next;

    while ((next = iterator.next()) && !next.done) {
      const array = next.value;
      const len = array.length;
      let i = 0;

      while (i < len) {
        const a = array[i];

        if (f(a)) {
          return true;
        }

        i++;
      }
    }

    return false;
  };
}
//# sourceMappingURL=forAny.js.map