"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.zipAllWith = zipAllWith;

var tsplus_module_1 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/Chunk/operations/empty"));

var tsplus_module_2 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/Chunk/operations/append"));

var _definition = /*#__PURE__*/require("@tsplus/stdlib/collections/Chunk/definition");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

/**
 * Zips with chunk with the specified chunk to produce a new chunk with
 * pairs of elements from each chunk combined using the specified function
 * `both`. If one chunk is shorter than the other uses the specified
 * function `left` or `right` to map the element that does exist to the
 * result type.
 *
 * @tsplus static Chunk.Aspects zipAllWith
 * @tsplus pipeable Chunk zipAllWith
 */
function zipAllWith(that, f, left, right) {
  return self => {
    const length = Math.max(self.length, that.length);

    if (length === 0) {
      return tsplus_module_1.empty();
    }

    const leftIterator = self._arrayLikeIterator();

    const rightIterator = that._arrayLikeIterator();

    let i = 0;
    let j = 0;
    let k = 0;
    let leftLength = 0;
    let rightLength = 0;
    let leftArray = undefined;
    let rightArray = undefined;
    let leftNext;
    let rightNext;
    let builder = tsplus_module_1.empty();

    while (i < length) {
      if (j < leftLength && k < rightLength) {
        builder = tsplus_module_2.append(f(leftArray[j], rightArray[k]))(builder);
        i++;
        j++;
        k++;
      } else if (j === leftLength && (leftNext = leftIterator.next()) && !leftNext.done) {
        leftArray = leftNext.value;
        leftLength = leftArray.length;
        j = 0;
      } else if (k === rightLength && (rightNext = rightIterator.next()) && !rightNext.done) {
        rightArray = rightNext.value;
        rightLength = rightArray.length;
        k = 0;
      } else if (j < leftLength) {
        builder = tsplus_module_2.append(left(leftArray[j]))(builder);
        i++;
        j++;
      } else if (k < rightLength) {
        builder = tsplus_module_2.append(right(rightArray[k]))(builder);
        i++;
        k++;
      }
    }

    return builder;
  };
}
//# sourceMappingURL=zipAllWith.js.map