import { Chunk } from "@tsplus/stdlib/collections/Chunk/definition";
export declare const ChunkPatchSym: unique symbol;
export type ChunkPatchSym = typeof ChunkPatchSym;
export declare const ChunkPatchValueSym: unique symbol;
export type ChunkPatchValueSym = typeof ChunkPatchValueSym;
export declare const ChunkPatchPatchSym: unique symbol;
export type ChunkPatchPatchSym = typeof ChunkPatchPatchSym;
/**
 * A patch which describes updates to a chunk of values.
 *
 * @tsplus type Differ.Chunk.Patch
 */
export interface ChunkPatch<in out Value, in out Patch> {
    readonly [ChunkPatchSym]: ChunkPatchSym;
    readonly [ChunkPatchValueSym]: (_: Value) => Value;
    readonly [ChunkPatchPatchSym]: (_: Patch) => Patch;
}
/**
 * @tsplus type Differ.Chunk.Patch.Ops
 */
export interface ChunkPatchOps {
    readonly $: ChunkPatchAspects;
}
/**
 * @tsplus static Differ.Ops Chunk
 * @tsplus location "@tsplus/stdlib/collections/Chunk/patch/definition"
 */
export declare const ChunkPatch: ChunkPatchOps;
/**
 * @tsplus type Differ.Chunk.Patch.Aspects
 */
export interface ChunkPatchAspects {
}
/**
 * @tsplus unify Differ.Chunk.Patch
 */
export declare function unifyChunkPatch<X extends ChunkPatch<any, any>>(self: X): ChunkPatch<[
    X
] extends [{
    [ChunkPatchValueSym]: (_: infer Value) => infer Value;
}] ? Value : never, [
    X
] extends [{
    [ChunkPatchPatchSym]: (_: infer Patch) => infer Patch;
}] ? Patch : never>;
export declare abstract class BaseChunkPatch<Value, Patch> implements ChunkPatch<Value, Patch> {
    readonly [ChunkPatchSym]: ChunkPatchSym;
    readonly [ChunkPatchValueSym]: (_: Value) => Value;
    readonly [ChunkPatchPatchSym]: (_: Patch) => Patch;
}
export declare class AppendChunkPatch<Value, Patch> extends BaseChunkPatch<Value, Patch> {
    readonly values: Chunk<Value>;
    readonly _tag = "Append";
    constructor(values: Chunk<Value>);
}
export declare class SliceChunkPatch<Value, Patch> extends BaseChunkPatch<Value, Patch> {
    readonly from: number;
    readonly until: number;
    readonly _tag = "Slice";
    constructor(from: number, until: number);
}
export declare class UpdateChunkPatch<Value, Patch> extends BaseChunkPatch<Value, Patch> {
    readonly index: number;
    readonly patch: Patch;
    readonly _tag = "Update";
    constructor(index: number, patch: Patch);
}
export declare class AndThenChunkPatch<Value, Patch> extends BaseChunkPatch<Value, Patch> {
    readonly first: ChunkPatch<Value, Patch>;
    readonly second: ChunkPatch<Value, Patch>;
    readonly _tag = "AndThen";
    constructor(first: ChunkPatch<Value, Patch>, second: ChunkPatch<Value, Patch>);
}
export declare class EmptyChunkPatch<Value, Patch> extends BaseChunkPatch<Value, Patch> {
    readonly _tag = "Empty";
}
export type ChunkPatchInstruction = AppendChunkPatch<any, any> | SliceChunkPatch<any, any> | UpdateChunkPatch<any, any> | AndThenChunkPatch<any, any> | EmptyChunkPatch<any, any>;
/**
 * @tsplus macro identity
 */
export declare function chunkPatchInstruction<Value, Patch>(self: ChunkPatch<Value, Patch>): ChunkPatchInstruction;
//# sourceMappingURL=definition.d.ts.map