"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.apply = apply;

var tsplus_module_1 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/data/Differ/definition"));

var tsplus_module_2 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/List/from"));

var tsplus_module_3 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/List/definition"));

var tsplus_module_4 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/Chunk/operations/concat"));

var tsplus_module_5 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/List/prepend"));

var tsplus_module_6 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/Collection/functions"));

var tsplus_module_7 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/Chunk/definition"));

var _definition4 = /*#__PURE__*/require("@tsplus/stdlib/collections/Chunk/patch/definition");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

/**
 * Applies a chunk patch to a chunk of values to produce a new chunk of
 * values which represents the original chunk of values updated with the
 * changes described by this patch.
 *
 * @tsplus static Differ.Chunk.Patch.Aspects apply
 * @tsplus pipeable Differ.Chunk.Patch apply
 */
function apply(oldValue, differ) {
  return self => applyLoop(differ, oldValue, tsplus_module_2.make(self));
}
/**
 * @tsplus tailRec
 */


function applyLoop(differ, chunk, patches) {
  var differ_1 = differ,
      chunk_1 = chunk,
      patches_1 = patches;
  var differ_2 = differ,
      chunk_2 = chunk,
      patches_2 = patches;

  while (1) {
    if (tsplus_module_3.isNil(patches_1)) {
      return chunk_1;
    }

    const patch = patches_1.head;
    const nextPatches = patches_1.tail;

    switch (patch._tag) {
      case "Append":
        {
          differ_2 = differ_1;
          chunk_2 = tsplus_module_4.concat(patch.values)(chunk_1);
          patches_2 = nextPatches;
          differ_1 = differ_2;
          chunk_1 = chunk_2;
          patches_1 = patches_2;
          continue;
        }

      case "AndThen":
        {
          differ_2 = differ_1;
          chunk_2 = chunk_1;
          patches_2 = tsplus_module_5.prepend(patch.first)(tsplus_module_5.prepend(patch.second)(nextPatches));
          differ_1 = differ_2;
          chunk_1 = chunk_2;
          patches_1 = patches_2;
          continue;
        }

      case "Empty":
        {
          differ_2 = differ_1;
          chunk_2 = chunk_1;
          patches_2 = nextPatches;
          differ_1 = differ_2;
          chunk_1 = chunk_2;
          patches_1 = patches_2;
          continue;
        }

      case "Slice":
        {
          differ_2 = differ_1;
          chunk_2 = tsplus_module_7.from(tsplus_module_6.toArray(chunk_1).slice(patch.from, patch.until));
          patches_2 = nextPatches;
          differ_1 = differ_2;
          chunk_1 = chunk_2;
          patches_1 = patches_2;
          continue;
        }

      case "Update":
        {
          const array = tsplus_module_6.toArray(chunk_1);
          array[patch.index] = differ_1.patch(patch.patch, array[patch.index]);
          differ_2 = differ_1;
          chunk_2 = tsplus_module_7.from(array);
          patches_2 = nextPatches;
          differ_1 = differ_2;
          chunk_1 = chunk_2;
          patches_1 = patches_2;
          continue;
        }
    }
  }
}
//# sourceMappingURL=apply.js.map