import { Collection } from "@tsplus/stdlib/collections/Collection/definition";
import { AssociativeIdentity } from "@tsplus/stdlib/prelude/AssociativeIdentity/definition";
/**
 * Zips the values of both iterators with the provided zipper function
 * @tsplus static Collection.Aspects zipWith
 * @tsplus pipeable Collection zipWith
 * @tsplus location "@tsplus/stdlib/collections/Collection/functions"
 */
export declare function zipWith<A, B, C>(that: Collection<B>, zipper: (a: A, b: B) => C): (self: Collection<A>) => Collection<C>;
/**
 * Maps the values of the iterator using the provided function
 * @tsplus static Collection.Aspects map
 * @tsplus pipeable Collection map
 * @tsplus location "@tsplus/stdlib/collections/Collection/functions"
 */
export declare function map<A, B>(f: (a: A, k: number) => B): (self: Collection<A>) => Collection<B>;
/**
 * Zips the two iterators into an iterator of a tuple
 * @tsplus static Collection.Aspects zip
 * @tsplus pipeable Collection zip
 * @tsplus location "@tsplus/stdlib/collections/Collection/functions"
 */
export declare function zip<B>(that: Collection<B>): <A>(self: import("./definition").Collection<A>) => import("./definition").Collection<readonly [A, B]>;
/**
 * Maps the iterator using the provided function and flatten its result
 * @tsplus static Collection.Aspects flatMap
 * @tsplus pipeable Collection flatMap
 * @tsplus location "@tsplus/stdlib/collections/Collection/functions"
 */
export declare function flatMap<A, B>(f: (a: A) => Collection<B>): (self: Collection<A>) => Collection<B>;
/**
 * Applicative's apply
 * @tsplus static Collection.Aspects ap
 * @tsplus pipeable Collection ap
 * @tsplus location "@tsplus/stdlib/collections/Collection/functions"
 */
export declare function ap<A, B>(fa: Collection<A>): (fab: import("./definition").Collection<(a: A) => B>) => Collection<B>;
/**
 * Creates an iterator of a single value
 * @tsplus static Collection.Ops of
 * @tsplus location "@tsplus/stdlib/collections/Collection/functions"
 */
export declare function of<A>(a: A): Collection<A>;
/**
 * Takes the fist n elements
 * @tsplus static Collection.Aspects take
 * @tsplus pipeable Collection take
 * @tsplus location "@tsplus/stdlib/collections/Collection/functions"
 */
export declare function take(n: number): <A>(self: import("./definition").Collection<A>) => import("./definition").Collection<A>;
/**
 * Skips the first n elements
 * @tsplus static Collection.Aspects skip
 * @tsplus pipeable Collection skip
 * @tsplus location "@tsplus/stdlib/collections/Collection/functions"
 */
export declare function skip(n: number): <A>(a: import("./definition").Collection<A>) => import("./definition").Collection<A>;
/**
 * Empty iterator
 * @tsplus static Collection.Ops never
 * @tsplus location "@tsplus/stdlib/collections/Collection/functions"
 */
export declare const never: Collection<never>;
/**
 * Loops over the iterator accumulating a result using the provided function giving access to the element index
 * @tsplus static Collection.Aspects reduceWithIndex
 * @tsplus pipeable Collection reduceWithIndex
 * @tsplus location "@tsplus/stdlib/collections/Collection/functions"
 */
export declare function reduceWithIndex<A, B>(b: B, f: (b: B, a: A, index: number) => B): (self: Collection<A>) => B;
/**
 * Loops over the iterator accumulating a result using the provided function
 * @tsplus static Collection.Aspects reduce
 * @tsplus pipeable Collection reduce
 * @tsplus location "@tsplus/stdlib/collections/Collection/functions"
 */
export declare function reduce<A, B>(b: B, f: (b: B, a: A) => B): (self: Collection<A>) => B;
/**
 * Loops over the iterator accumulating a result using the provided function and AssociativeIdentity giving access to the element index
 * @tsplus static Collection.Aspects foldMapWithIndex
 * @tsplus pipeable Collection foldMapWithIndex
 * @tsplus location "@tsplus/stdlib/collections/Collection/functions"
 */
export declare function foldMapWithIndex<M, A>(M: AssociativeIdentity<M>, f: (a: A, index: number) => M): (self: Collection<A>) => M;
/**
 * Loops over the iterator accumulating a result using the provided function and AssociativeIdentity
 * @tsplus static Collection.Aspects foldMap
 * @tsplus pipeable Collection foldMap
 * @tsplus location "@tsplus/stdlib/collections/Collection/functions"
 */
export declare function foldMap<M, A>(M: AssociativeIdentity<M>, f: (a: A) => M): (self: Collection<A>) => M;
/**
 * Concats iterators together
 * @tsplus pipeable-operator Collection &
 * @tsplus static Collection.Aspects concat
 * @tsplus pipeable Collection concat
 * @tsplus location "@tsplus/stdlib/collections/Collection/functions"
 */
export declare function concat<B>(that: Collection<B>): <A>(self: import("./definition").Collection<A>) => import("./definition").Collection<B | A>;
/**
 * Concats iterators together that are strictly of the same type
 * @tsplus pipeable-operator Collection +
 * @tsplus location "@tsplus/stdlib/collections/Collection/functions"
 */
export declare function concatOperator<A>(that: Collection<A>): (self: Collection<A>) => Collection<A>;
/**
 * Prepends a value to an iterator
 * @tsplus static Collection.Aspects prepend
 * @tsplus pipeable Collection prepend
 * @tsplus location "@tsplus/stdlib/collections/Collection/functions"
 */
export declare function prepend<B>(that: B): <A>(self: import("./definition").Collection<A>) => import("./definition").Collection<B | A>;
/**
 * Prepends a value to an iterator
 * @tsplus pipeable-operator Collection >
 * @tsplus location "@tsplus/stdlib/collections/Collection/functions"
 */
export declare function prependOperator<B>(self: Collection<B>): <A>(a: A) => import("./definition").Collection<B | A>;
/**
 * Prepends a value to an iterator of the same type
 * @tsplus pipeable-operator Collection + 1.0
 * @tsplus location "@tsplus/stdlib/collections/Collection/functions"
 */
export declare function prependOperatorStrict<A>(self: Collection<A>): (a: A) => Collection<A>;
/**
 * Appends a value to an iterator.
 * @tsplus pipeable-operator Collection <
 * @tsplus static Collection.Aspects append
 * @tsplus pipeable Collection append
 * @tsplus location "@tsplus/stdlib/collections/Collection/functions"
 */
export declare function append<B>(that: B): <A>(self: import("./definition").Collection<A>) => import("./definition").Collection<B | A>;
/**
 * Appends a value to an iterator of the same type
 * @tsplus pipeable-operator Collection + 1.0
 * @tsplus location "@tsplus/stdlib/collections/Collection/functions"
 */
export declare function appendOperatorStrict<A>(a: A): (self: Collection<A>) => Collection<A>;
/**
 * Flattens nested iterators
 * @tsplus getter Collection flatten
 * @tsplus location "@tsplus/stdlib/collections/Collection/functions"
 */
export declare function flatten<A>(self: Collection<Collection<A>>): import("./definition").Collection<A>;
/**
 * Infinite sequence produced by repeated application of f to a
 * @tsplus static Collection.Ops unfold
 * @tsplus location "@tsplus/stdlib/collections/Collection/functions"
 */
export declare function unfold<A>(a: A, f: (a: A) => A): Collection<A>;
/**
 * Compares each element of the iterators using the provided function
 * @tsplus static Collection.Aspects equalsWith
 * @tsplus pipeable Collection equalsWith
 * @tsplus location "@tsplus/stdlib/collections/Collection/functions"
 */
export declare function equalsWith<A, B>(that: Collection<B>, f: (a: A, b: B) => boolean): (self: Collection<A>) => boolean;
/**
 * Compares the iterators using value equality
 * @tsplus pipeable-operator Collection ==
 * @tsplus static Collection.Aspects equals
 * @tsplus pipeable Collection equals
 * @tsplus location "@tsplus/stdlib/collections/Collection/functions"
 */
export declare function equals<A>(that: Collection<A>): (self: Collection<A>) => boolean;
export declare function equals<B>(that: Collection<B>): <A>(self: Collection<A>) => boolean;
/**
 * @tsplus static Collection.Ops make
 * @tsplus static Collection.Ops __call
 * @tsplus location "@tsplus/stdlib/collections/Collection/functions"
 */
export declare function make<A extends readonly any[]>(...as: A): Collection<A[number]>;
/**
 * @tsplus getter Collection toArray
 * @tsplus location "@tsplus/stdlib/collections/Collection/functions"
 */
export declare function toArray<A>(data: Collection<A>): Array<A>;
//# sourceMappingURL=functions.d.ts.map