import { HashMap } from "@tsplus/stdlib/collections/HashMap/definition";
import { Hash } from "@tsplus/stdlib/structure/Hash";
import { Equals } from "@tsplus/stdlib/structure/Equals";
import { Maybe } from "@tsplus/stdlib/data/Maybe/definition";
import type { Node } from "@tsplus/stdlib/collections/HashMap/_internal/node";
import { _K, _V, HashMapSym } from "@tsplus/stdlib/collections/HashMap/definition";
export type TraversalFn<K, V, A> = (k: K, v: V) => A;
export type Cont<K, V, A> = [
    len: number,
    children: Node<K, V>[],
    i: number,
    f: TraversalFn<K, V, A>,
    cont: Cont<K, V, A>
] | undefined;
export interface VisitResult<K, V, A> {
    value: A;
    cont: Cont<K, V, A>;
}
export declare class HashMapInternal<K, V> implements HashMap<K, V> {
    _editable: boolean;
    _edit: number;
    _root: Node<K, V>;
    _size: number;
    readonly [HashMapSym]: HashMapSym;
    readonly [_K]: () => K;
    readonly [_V]: () => V;
    constructor(_editable: boolean, _edit: number, _root: Node<K, V>, _size: number);
    [Symbol.iterator](): Iterator<readonly [K, V]>;
    [Hash.sym](): number;
    [Equals.sym](that: unknown): boolean;
}
export declare class HashMapIterator<K, V, T> implements IterableIterator<T> {
    readonly map: HashMapInternal<K, V>;
    readonly f: TraversalFn<K, V, T>;
    v: import("../../../data/Maybe").Maybe<VisitResult<K, V, T>>;
    constructor(map: HashMapInternal<K, V>, f: TraversalFn<K, V, T>);
    next(): IteratorResult<T>;
    [Symbol.iterator](): IterableIterator<T>;
}
/**
 * @tsplus macro remove
 */
export declare function realHashMap<K, V>(_: HashMap<K, V>): asserts _ is HashMapInternal<K, V>;
export declare function applyCont<K, V, A>(cont: Cont<K, V, A>): import("../../../data/Maybe").Maybe<VisitResult<K, V, A>>;
export declare function visitLazy<K, V, A>(node: Node<K, V>, f: TraversalFn<K, V, A>, cont?: Cont<K, V, A>): Maybe<VisitResult<K, V, A>>;
export declare function visitLazyChildren<K, V, A>(len: number, children: Node<K, V>[], i: number, f: TraversalFn<K, V, A>, cont: Cont<K, V, A>): Maybe<VisitResult<K, V, A>>;
/**
 * Lookup the value for the specified key in the `HashMap` using a custom hash.
 * @tsplus fluent HashMap getHash
 * @tsplus location "@tsplus/stdlib/collections/HashMap/_internal/hashMap"
 */
export declare function getHash_<K, V>(self: HashMap<K, V>, key: K, hash: number): Maybe<V>;
/**
 * Lookup the value for the specified key in the `HashMap` using a custom hash.
 * @tsplus static HashMap.Aspects getHash
 * @tsplus pipeable HashMap getHash
 * @tsplus location "@tsplus/stdlib/collections/HashMap/_internal/hashMap"
 */
export declare const getHash: <K>(key: K, hash: number) => <V>(self: import("@tsplus/stdlib/collections/HashMap/definition").HashMap<K, V>) => import("../../../data/Maybe").Maybe<V>;
/**
 * Checks if the specified key has an entry in the `HashMap`.
 * @tsplus fluent HashMap has
 * @tsplus location "@tsplus/stdlib/collections/HashMap/_internal/hashMap"
 */
export declare function has_<K, V>(self: HashMap<K, V>, key: K): boolean;
/**
 * Checks if the specified key has an entry in the `HashMap`.
 * @tsplus static HashMap.Aspects has
 * @tsplus pipeable HashMap has
 * @tsplus location "@tsplus/stdlib/collections/HashMap/_internal/hashMap"
 */
export declare const has: <K>(key: K) => <V>(self: import("@tsplus/stdlib/collections/HashMap/definition").HashMap<K, V>) => boolean;
/**
 * @tsplus operator HashMap ==
 * @tsplus fluent HashMap equals
 * @tsplus location "@tsplus/stdlib/collections/HashMap/_internal/hashMap"
 */
export declare function equals_<K, V>(self: HashMap<K, V>, that: HashMap<K, V>): boolean;
export declare function equals_<K, V, K1, V1>(self: HashMap<K, V>, that: HashMap<K1, V1>): boolean;
/**
 * @tsplus pipeable HashMap equals
 * @tsplus location "@tsplus/stdlib/collections/HashMap/_internal/hashMap"
 */
export declare const equals: <K, V>(that: import("@tsplus/stdlib/collections/HashMap/definition").HashMap<K, V>) => (self: import("@tsplus/stdlib/collections/HashMap/definition").HashMap<K, V>) => boolean;
//# sourceMappingURL=hashMap.d.ts.map