"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.HashMapIterator = exports.HashMapInternal = void 0;
exports.applyCont = applyCont;
exports.equals = void 0;
exports.equals_ = equals_;
exports.has = exports.getHash_ = exports.getHash = void 0;
exports.has_ = has_;
exports.realHashMap = realHashMap;
exports.visitLazy = visitLazy;
exports.visitLazyChildren = visitLazyChildren;

var tsplus_module_1 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/structure/Hash"));

var tsplus_module_2 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/structure/Equals"));

var tsplus_module_3 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/HashMap/definition"));

var tsplus_module_4 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/data/Maybe/definition"));

var _bitwise = /*#__PURE__*/require("@tsplus/stdlib/collections/HashMap/_internal/bitwise");

var _config = /*#__PURE__*/require("@tsplus/stdlib/collections/HashMap/_internal/config");

var _node = /*#__PURE__*/require("@tsplus/stdlib/collections/HashMap/_internal/node");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

var _a;

const getHash_ = getHash_1;
exports.getHash_ = getHash_;

class HashMapInternal {
  constructor(_editable, _edit, _root, _size) {
    this._editable = _editable;
    this._edit = _edit;
    this._root = _root;
    this._size = _size;
    this[_a] = tsplus_module_3.HashMapSym;
  }

  [(_a = tsplus_module_3.HashMapSym, Symbol.iterator)]() {
    return new HashMapIterator(this, (k, v) => [k, v]);
  }

  [tsplus_module_1.Hash.sym]() {
    let hash = tsplus_module_1.hashString("HashMap");

    for (const item of this) {
      hash ^= tsplus_module_1.combine(tsplus_module_1.hashUnknown(item[0]), tsplus_module_1.hashUnknown(item[1]));
    }

    return tsplus_module_1.optimize(hash);
  }

  [tsplus_module_2.Equals.sym](that) {
    if (tsplus_module_3.isHashMap(that)) {
      void 0;

      if (that._size !== this._size) {
        return false;
      }

      for (const item of this) {
        const elem = getHash_1(that, item[0], tsplus_module_1.hashUnknown(item[0]));

        if (tsplus_module_4.isNone(elem)) {
          return false;
        } else {
          if (!tsplus_module_2.equals(item[1], elem.value)) {
            return false;
          }
        }
      }

      return true;
    }

    return false;
  }

}

exports.HashMapInternal = HashMapInternal;

class HashMapIterator {
  constructor(map, f) {
    this.map = map;
    this.f = f;
    this.v = visitLazy(this.map._root, this.f, undefined);
  }

  next() {
    if (tsplus_module_4.isNone(this.v)) {
      return {
        done: true,
        value: undefined
      };
    }

    const v0 = this.v.value;
    this.v = applyCont(v0.cont);
    return {
      done: false,
      value: v0.value
    };
  }

  [Symbol.iterator]() {
    return new HashMapIterator(this.map, this.f);
  }

}
/**
 * @tsplus macro remove
 */


exports.HashMapIterator = HashMapIterator;

function realHashMap(_) {//
}

function applyCont(cont) {
  return cont ? visitLazyChildren(cont[0], cont[1], cont[2], cont[3], cont[4]) : tsplus_module_4.none;
}

function visitLazy(node, f, cont = undefined) {
  switch (node._tag) {
    case "LeafNode":
      {
        return tsplus_module_4.isSome(node.value) ? tsplus_module_4.some({
          value: f(node.key, node.value.value),
          cont
        }) : applyCont(cont);
      }

    case "CollisionNode":
    case "ArrayNode":
    case "IndexedNode":
      {
        const children = node.children;
        return visitLazyChildren(children.length, children, 0, f, cont);
      }

    default:
      {
        return applyCont(cont);
      }
  }
}

function visitLazyChildren(len, children, i, f, cont) {
  while (i < len) {
    const child = children[i++];

    if (child && !(0, _node.isEmptyNode)(child)) {
      return visitLazy(child, f, [len, children, i, f, cont]);
    }
  }

  return applyCont(cont);
}
/**
 * Lookup the value for the specified key in the `HashMap` using a custom hash.
 *
 * @tsplus fluent HashMap getHash
 */


function getHash_1(self, key, hash) {
  void 0;
  let node = self._root;
  let shift = 0; // eslint-disable-next-line no-constant-condition

  while (true) {
    switch (node._tag) {
      case "LeafNode":
        {
          return tsplus_module_2.equals(key, node.key) ? node.value : tsplus_module_4.none;
        }

      case "CollisionNode":
        {
          if (hash === node.hash) {
            const children = node.children;

            for (let i = 0, len = children.length; i < len; ++i) {
              const child = children[i];
              if ("key" in child && tsplus_module_2.equals(key, child.key)) return child.value;
            }
          }

          return tsplus_module_4.none;
        }

      case "IndexedNode":
        {
          const frag = (0, _bitwise.hashFragment)(shift, hash);
          const bit = (0, _bitwise.toBitmap)(frag);

          if (node.mask & bit) {
            node = node.children[(0, _bitwise.fromBitmap)(node.mask, bit)];
            shift += _config.SIZE;
            break;
          }

          return tsplus_module_4.none;
        }

      case "ArrayNode":
        {
          node = node.children[(0, _bitwise.hashFragment)(shift, hash)];

          if (node) {
            shift += _config.SIZE;
            break;
          }

          return tsplus_module_4.none;
        }

      default:
        return tsplus_module_4.none;
    }
  }
}
/**
 * Lookup the value for the specified key in the `HashMap` using a custom hash.
 *
 * @tsplus static HashMap.Aspects getHash
 */


const getHash = (key, hash) => self => getHash_(self, key, hash);
/**
 * Checks if the specified key has an entry in the `HashMap`.
 *
 * @tsplus fluent HashMap has
 */


exports.getHash = getHash;

function has_(self, key) {
  return tsplus_module_4.isSome(getHash_1(self, key, tsplus_module_1.hashUnknown(key)));
}
/**
 * Checks if the specified key has an entry in the `HashMap`.
 *
 * @tsplus static HashMap.Aspects has
 */


const has = key => self => has_(self, key);

exports.has = has;

function equals_(self, that) {
  return tsplus_module_2.equals(self, that);
}

const equals = that => self => equals_(self, that);

exports.equals = equals;
//# sourceMappingURL=hashMap.js.map