import { Maybe } from "@tsplus/stdlib/data/Maybe/definition";
import { Equivalence } from "@tsplus/stdlib/prelude/Equivalence/definition";
import { Equals } from "@tsplus/stdlib/structure/Equals";
import { Stack } from "@tsplus/stdlib/data/Stack";
/**
 * @tsplus type HashMap.Node
 */
export type Node<K, V> = EmptyNode<K, V> | LeafNode<K, V> | CollisionNode<K, V> | IndexedNode<K, V> | ArrayNode<K, V>;
/**
 * @tsplus type HashMap.Node.Ops
 */
export interface NodeOps {
}
export declare const Node: NodeOps;
export interface SizeRef {
    value: number;
}
export declare class EmptyNode<K, V> {
    readonly _tag = "EmptyNode";
    modify(edit: number, _shift: number, f: UpdateFn<V>, hash: number, key: K, size: SizeRef): Node<K, V>;
}
export declare function isEmptyNode(a: unknown): a is EmptyNode<unknown, unknown>;
export declare function isLeafNode<K, V>(node: Node<K, V>): node is EmptyNode<K, V> | LeafNode<K, V> | CollisionNode<K, V>;
export declare function canEditNode<K, V>(node: Node<K, V>, edit: number): boolean;
export type KeyEq<K> = Equivalence<K>["equals"];
export type UpdateFn<V> = (v: Maybe<V>) => Maybe<V>;
export declare class LeafNode<K, V> {
    readonly edit: number;
    readonly hash: number;
    readonly key: K;
    value: Maybe<V>;
    readonly _tag = "LeafNode";
    constructor(edit: number, hash: number, key: K, value: Maybe<V>);
    modify(edit: number, shift: number, f: UpdateFn<V>, hash: number, key: K, size: SizeRef): Node<K, V>;
}
export declare class CollisionNode<K, V> {
    readonly edit: number;
    readonly hash: number;
    readonly children: Array<Node<K, V>>;
    readonly _tag = "CollisionNode";
    constructor(edit: number, hash: number, children: Array<Node<K, V>>);
    modify(edit: number, shift: number, f: UpdateFn<V>, hash: number, key: K, size: SizeRef): Node<K, V>;
    updateCollisionList(mutate: boolean, edit: number, hash: number, list: Node<K, V>[], f: UpdateFn<V>, key: K, size: SizeRef): Node<K, V>[];
}
export declare class IndexedNode<K, V> {
    readonly edit: number;
    mask: number;
    children: Array<Node<K, V>>;
    readonly _tag = "IndexedNode";
    constructor(edit: number, mask: number, children: Array<Node<K, V>>);
    modify(edit: number, shift: number, f: UpdateFn<V>, hash: number, key: K, size: SizeRef): Node<K, V>;
}
export declare class ArrayNode<K, V> {
    readonly edit: number;
    size: number;
    children: Array<Node<K, V>>;
    readonly _tag = "ArrayNode";
    constructor(edit: number, size: number, children: Array<Node<K, V>>);
    modify(edit: number, shift: number, f: UpdateFn<V>, hash: number, key: K, size: SizeRef): Node<K, V>;
}
//# sourceMappingURL=node.d.ts.map