import { Maybe } from "@tsplus/stdlib/data/Maybe/definition";
import type { Cont, TraversalFn, VisitResult } from "@tsplus/stdlib/collections/HashMap/_internal/hashMap";
import { visitLazyChildren as visitLazyChildrenInternal } from "@tsplus/stdlib/collections/HashMap/_internal/hashMap";
import type { Node } from "@tsplus/stdlib/collections/HashMap/_internal/node";
import { isEmptyNode as isEmptyNodeInternal, isLeafNode as isLeafNodeInternal } from "@tsplus/stdlib/collections/HashMap/_internal/node";
export { Node } from "@tsplus/stdlib/collections/HashMap/_internal/node";
/**
 * @tsplus static HashMap.Node.Ops empty
 * @tsplus location "@tsplus/stdlib/collections/HashMap/operations/node"
 */
export declare function emptyNode<K = never, V = never>(): Node<K, V>;
/**
 * @tsplus static HashMap.Node.Ops leaf
 * @tsplus location "@tsplus/stdlib/collections/HashMap/operations/node"
 */
export declare function leafNode<K, V>(edit: number, hash: number, key: K, value: Maybe<V>): Node<K, V>;
/**
 * @tsplus static HashMap.Node.Ops collision
 * @tsplus location "@tsplus/stdlib/collections/HashMap/operations/node"
 */
export declare function collisionNode<K, V>(edit: number, hash: number, children: Array<Node<K, V>>): Node<K, V>;
/**
 * @tsplus static HashMap.Node.Ops indexed
 * @tsplus location "@tsplus/stdlib/collections/HashMap/operations/node"
 */
export declare function indexedNode<K, V>(edit: number, mask: number, children: Array<Node<K, V>>): Node<K, V>;
/**
 * @tsplus static HashMap.Node.Ops array
 * @tsplus location "@tsplus/stdlib/collections/HashMap/operations/node"
 */
export declare function arrayNode<K, V>(edit: number, size: number, children: Array<Node<K, V>>): Node<K, V>;
/**
 * @tsplus static HashMap.Node.Ops isEmptyNode
 * @tsplus location "@tsplus/stdlib/collections/HashMap/operations/node"
 */
export declare const isEmptyNode: typeof isEmptyNodeInternal;
/**
 * @tsplus static HashMap.Node.Ops isLeafNode
 * @tsplus location "@tsplus/stdlib/collections/HashMap/operations/node"
 */
export declare const isLeafNode: typeof isLeafNodeInternal;
/**
 * @tsplus pipeable HashMap.Node canEdit
 * @tsplus location "@tsplus/stdlib/collections/HashMap/operations/node"
 */
export declare function canEditNode(edit: number): <K, V>(node: Node<K, V>) => boolean;
/**
 * @tsplus pipeable HashMap.Node visitLazy
 * @tsplus location "@tsplus/stdlib/collections/HashMap/operations/node"
 */
export declare function visitLazy<K, V, A>(f: TraversalFn<K, V, A>, cont?: Cont<K, V, A>): (node: Node<K, V>) => Maybe<VisitResult<K, V, A>>;
/**
 * @tsplus fluent HashMap.Node.Ops visitLazyChildren
 * @tsplus location "@tsplus/stdlib/collections/HashMap/operations/node"
 */
export declare const visitLazyChildren: typeof visitLazyChildrenInternal;
//# sourceMappingURL=node.d.ts.map