export declare const HashMapPatchSym: unique symbol;
export type HashMapPatchSym = typeof HashMapPatchSym;
export declare const HashMapPatchKeySym: unique symbol;
export type HashMapPatchKeySym = typeof HashMapPatchKeySym;
export declare const HashMapPatchValueSym: unique symbol;
export type HashMapPatchValueSym = typeof HashMapPatchValueSym;
export declare const HashMapPatchPatchSym: unique symbol;
export type HashMapPatchPatchSym = typeof HashMapPatchPatchSym;
/**
 * A patch which describes updates to a map of keys and values.
 *
 * @tsplus type Differ.HashMap.Patch
 */
export interface HashMapPatch<in out Key, in out Value, in out Patch> {
    readonly [HashMapPatchSym]: HashMapPatchSym;
    readonly [HashMapPatchKeySym]: (_: Key) => Key;
    readonly [HashMapPatchValueSym]: (_: Value) => Value;
    readonly [HashMapPatchPatchSym]: (_: Patch) => Patch;
}
/**
 * @tsplus type Differ.HashMap.Patch.Ops
 */
export interface HashMapPatchOps {
    readonly $: HashMapPatchAspects;
}
/**
 * @tsplus static Differ.Ops HashMap
 * @tsplus location "@tsplus/stdlib/collections/HashMap/patch/definition"
 */
export declare const HashMapPatch: HashMapPatchOps;
/**
 * @tsplus type Differ.HashMap.Patch.Aspects
 */
export interface HashMapPatchAspects {
}
/**
 * @tsplus unify Differ.HashMap.Patch
 */
export declare function unifyHashMapPatch<X extends HashMapPatch<any, any, any>>(self: X): HashMapPatch<[
    X
] extends [{
    [HashMapPatchKeySym]: (_: infer Key) => infer Key;
}] ? Key : never, [
    X
] extends [{
    [HashMapPatchValueSym]: (_: infer Value) => infer Value;
}] ? Value : never, [
    X
] extends [{
    [HashMapPatchPatchSym]: (_: infer Patch) => infer Patch;
}] ? Patch : never>;
export declare abstract class BaseHashMapPatch<Key, Value, Patch> implements HashMapPatch<Key, Value, Patch> {
    readonly [HashMapPatchSym]: HashMapPatchSym;
    readonly [HashMapPatchKeySym]: (_: Key) => Key;
    readonly [HashMapPatchValueSym]: (_: Value) => Value;
    readonly [HashMapPatchPatchSym]: (_: Patch) => Patch;
}
export declare class AddHashMapPatch<Key, Value, Patch> extends BaseHashMapPatch<Key, Value, Patch> {
    readonly key: Key;
    readonly value: Value;
    readonly _tag = "Add";
    constructor(key: Key, value: Value);
}
export declare class RemoveHashMapPatch<Key, Value, Patch> extends BaseHashMapPatch<Key, Value, Patch> {
    readonly key: Key;
    readonly _tag = "Remove";
    constructor(key: Key);
}
export declare class UpdateHashMapPatch<Key, Value, Patch> extends BaseHashMapPatch<Key, Value, Patch> {
    readonly key: Key;
    readonly patch: Patch;
    readonly _tag = "Update";
    constructor(key: Key, patch: Patch);
}
export declare class EmptyHashMapPatch<Key, Value, Patch> extends BaseHashMapPatch<Key, Value, Patch> {
    readonly _tag = "Empty";
}
export declare class AndThenHashMapPatch<Key, Value, Patch> extends BaseHashMapPatch<Key, Value, Patch> {
    readonly first: HashMapPatch<Key, Value, Patch>;
    readonly second: HashMapPatch<Key, Value, Patch>;
    readonly _tag = "AndThen";
    constructor(first: HashMapPatch<Key, Value, Patch>, second: HashMapPatch<Key, Value, Patch>);
}
export type HashMapPatchInstruction = AddHashMapPatch<any, any, any> | RemoveHashMapPatch<any, any, any> | UpdateHashMapPatch<any, any, any> | EmptyHashMapPatch<any, any, any> | AndThenHashMapPatch<any, any, any>;
/**
 * @tsplus macro identity
 */
export declare function hashMapPatchInstruction<Key, Value, Patch>(self: HashMapPatch<Key, Value, Patch>): HashMapPatchInstruction;
//# sourceMappingURL=definition.d.ts.map