"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.apply = apply;

var tsplus_module_1 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/data/Differ/definition"));

var tsplus_module_2 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/List/from"));

var tsplus_module_3 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/List/definition"));

var tsplus_module_4 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/HashMap/operations/set"));

var tsplus_module_5 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/List/prepend"));

var tsplus_module_6 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/HashMap/operations/remove"));

var tsplus_module_7 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/HashMap/operations/get"));

var tsplus_module_8 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/data/Maybe/fold"));

var _definition3 = /*#__PURE__*/require("@tsplus/stdlib/collections/HashMap/patch/definition");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

/**
 * Applies a map patch to a map of keys and values to produce a new map of
 * keys and values values which represents the original map of keys and
 * values updated with the changes described by this patch.
 *
 * @tsplus static Differ.HashMap.Patch.Aspects apply
 * @tsplus pipeable Differ.HashMap.Patch apply
 */
function apply(oldValue, differ) {
  return self => applyLoop(differ, oldValue, tsplus_module_2.make(self));
}
/**
 * @tsplus tailRec
 */


function applyLoop(differ, map, patches) {
  var differ_1 = differ,
      map_1 = map,
      patches_1 = patches;
  var differ_2 = differ,
      map_2 = map,
      patches_2 = patches;

  while (1) {
    if (tsplus_module_3.isNil(patches_1)) {
      return map_1;
    }

    const patch = patches_1.head;
    const nextPatches = patches_1.tail;

    switch (patch._tag) {
      case "Add":
        {
          differ_2 = differ_1;
          map_2 = tsplus_module_4.set(patch.key, patch.value)(map_1);
          patches_2 = nextPatches;
          differ_1 = differ_2;
          map_1 = map_2;
          patches_1 = patches_2;
          continue;
        }

      case "AndThen":
        {
          differ_2 = differ_1;
          map_2 = map_1;
          patches_2 = tsplus_module_5.prepend(patch.first)(tsplus_module_5.prepend(patch.second)(nextPatches));
          differ_1 = differ_2;
          map_1 = map_2;
          patches_1 = patches_2;
          continue;
        }

      case "Empty":
        {
          differ_2 = differ_1;
          map_2 = map_1;
          patches_2 = nextPatches;
          differ_1 = differ_2;
          map_1 = map_2;
          patches_1 = patches_2;
          continue;
        }

      case "Remove":
        {
          differ_2 = differ_1;
          map_2 = tsplus_module_6.remove(patch.key)(map_1);
          patches_2 = nextPatches;
          differ_1 = differ_2;
          map_1 = map_2;
          patches_1 = patches_2;
          continue;
        }

      case "Update":
        {
          differ_2 = differ_1;
          map_2 = tsplus_module_8.fold(() => map_1, oldValue => tsplus_module_4.set(patch.key, differ_1.patch(patch.patch, oldValue))(map_1))(tsplus_module_7.get(patch.key)(map_1));
          patches_2 = nextPatches;
          differ_1 = differ_2;
          map_1 = map_2;
          patches_1 = patches_2;
          continue;
        }
    }
  }
}
//# sourceMappingURL=apply.js.map