export declare const HashSetPatchSym: unique symbol;
export type HashSetPatchSym = typeof HashSetPatchSym;
export declare const HashSetPatchValueSym: unique symbol;
export type HashSetPatchValueSym = typeof HashSetPatchValueSym;
/**
 * A patch which describes updates to a set of values.
 *
 * @tsplus type Differ.HashSet.Patch
 */
export interface HashSetPatch<in out Value> {
    readonly [HashSetPatchSym]: HashSetPatchSym;
    readonly [HashSetPatchValueSym]: (_: Value) => Value;
}
/**
 * @tsplus type Differ.HashSet.Patch.Ops
 */
export interface HashSetPatchOps {
    readonly $: HashSetPatchAspects;
}
/**
 * @tsplus static Differ.Ops HashSet
 * @tsplus location "@tsplus/stdlib/collections/HashSet/patch/definition"
 */
export declare const HashSetPatch: HashSetPatchOps;
/**
 * @tsplus type Differ.HashSet.Patch.Aspects
 */
export interface HashSetPatchAspects {
}
/**
 * @tsplus unify Differ.HashSet.Patch
 */
export declare function unifyHashSetPatch<X extends HashSetPatch<any>>(self: X): HashSetPatch<[
    X
] extends [{
    [HashSetPatchValueSym]: (_: infer Value) => infer Value;
}] ? Value : never>;
export declare abstract class BaseHashSetPatch<Value> implements HashSetPatch<Value> {
    readonly [HashSetPatchSym]: HashSetPatchSym;
    readonly [HashSetPatchValueSym]: (_: Value) => Value;
}
export declare class AddHashSetPatch<Value> extends BaseHashSetPatch<Value> {
    readonly value: Value;
    readonly _tag = "Add";
    constructor(value: Value);
}
export declare class AndThenHashSetPatch<Value> extends BaseHashSetPatch<Value> {
    readonly first: HashSetPatch<Value>;
    readonly second: HashSetPatch<Value>;
    readonly _tag = "AndThen";
    constructor(first: HashSetPatch<Value>, second: HashSetPatch<Value>);
}
export declare class EmptyHashSetPatch<Value> extends BaseHashSetPatch<Value> {
    readonly _tag = "Empty";
}
export declare class RemoveHashSetPatch<Value> extends BaseHashSetPatch<Value> {
    readonly value: Value;
    readonly _tag = "Remove";
    constructor(value: Value);
}
export type HashSetPatchInstruction = AddHashSetPatch<any> | AndThenHashSetPatch<any> | EmptyHashSetPatch<any> | RemoveHashSetPatch<any>;
/**
 * @tsplus macro identity
 */
export declare function hashSetPatchInstruction<Value>(self: HashSetPatch<Value>): HashSetPatchInstruction;
//# sourceMappingURL=definition.d.ts.map