"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.apply = apply;

var tsplus_module_1 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/data/Differ/definition"));

var tsplus_module_2 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/List/from"));

var tsplus_module_3 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/List/definition"));

var tsplus_module_4 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/HashSet/operations/add"));

var tsplus_module_5 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/List/prepend"));

var tsplus_module_6 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/HashSet/operations/remove"));

var _definition3 = /*#__PURE__*/require("@tsplus/stdlib/collections/HashSet/patch/definition");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

/**
 * Applies a set patch to a set of values to produce a new set of values
 * which represents the original set of values updated with the changes
 * described by this patch.
 *
 * @tsplus static Differ.HashSet.Patch.Aspects apply
 * @tsplus pipeable Differ.HashSet.Patch apply
 */
function apply(oldValue) {
  return self => applyLoop(oldValue, tsplus_module_2.make(self));
}
/**
 * @tsplus tailRec
 */


function applyLoop(set, patches) {
  var set_1 = set,
      patches_1 = patches;
  var set_2 = set,
      patches_2 = patches;

  while (1) {
    if (tsplus_module_3.isNil(patches_1)) {
      return set_1;
    }

    const patch = patches_1.head;
    const nextPatches = patches_1.tail;

    switch (patch._tag) {
      case "Add":
        {
          set_2 = tsplus_module_4.add(patch.value)(set_1);
          patches_2 = nextPatches;
          set_1 = set_2;
          patches_1 = patches_2;
          continue;
        }

      case "AndThen":
        {
          set_2 = set_1;
          patches_2 = tsplus_module_5.prepend(patch.first)(tsplus_module_5.prepend(patch.second)(nextPatches));
          set_1 = set_2;
          patches_1 = patches_2;
          continue;
        }

      case "Empty":
        {
          set_2 = set_1;
          patches_2 = nextPatches;
          set_1 = set_2;
          patches_1 = patches_2;
          continue;
        }

      case "Remove":
        {
          set_2 = tsplus_module_6.remove(patch.value)(set_1);
          patches_2 = nextPatches;
          set_1 = set_2;
          patches_1 = patches_2;
          continue;
        }
    }
  }
}
//# sourceMappingURL=apply.js.map