"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.lt = lt;

var _definition = /*#__PURE__*/require("@tsplus/stdlib/collections/RedBlackTree/definition");

/**
 * Returns an iterator that traverse entries with keys less than the specified
 * key.
 *
 * @tsplus static RedBlackTree.Aspects lt
 * @tsplus pipeable RedBlackTree lt
 */
function lt(key, direction = "Forward") {
  return self => {
    return {
      ord: self.ord,
      [Symbol.iterator]: () => {
        const cmp = self.ord.compare;
        let n = self.root;
        const stack = [];
        let last_ptr = 0;

        while (n) {
          const d = cmp(key, n.key);
          stack.push(n);

          if (d > 0) {
            last_ptr = stack.length;
          }

          if (d <= 0) {
            n = n.left;
          } else {
            n = n.right;
          }
        }

        stack.length = last_ptr;
        return new _definition.RedBlackTreeIterator(self, stack, direction);
      }
    };
  };
}
//# sourceMappingURL=lt.js.map