"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Node = void 0;
exports.clone = clone;
exports.recount = recount;
exports.repaint = repaint;
exports.swap = swap;

/**
 * @tsplus type RedBlackTree.Node
 * @tsplus companion RedBlackTree.Node.Ops
 */
class Node {
  constructor(color, key, value, left, right, count) {
    this.color = color;
    this.key = key;
    this.value = value;
    this.left = left;
    this.right = right;
    this.count = count;
  }

}
/**
 * @tsplus fluent RedBlackTree.Node clone
 */


exports.Node = Node;

function clone(node) {
  return new Node(node.color, node.key, node.value, node.left, node.right, node.count);
}
/**
 * @tsplus fluent RedBlackTree.Node swap
 */


function swap(n, v) {
  n.key = v.key;
  n.value = v.value;
  n.left = v.left;
  n.right = v.right;
  n.color = v.color;
  n.count = v.count;
}
/**
 * @tsplus fluent RedBlackTree.Node repaint
 */


function repaint(node, color) {
  return new Node(color, node.key, node.value, node.left, node.right, node.count);
}
/**
 * @tsplus fluent RedBlackTree.Node recount
 */


function recount(node) {
  node.count = 1 + (node.left?.count ?? 0) + (node.right?.count ?? 0);
}
//# sourceMappingURL=node.js.map