"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.LinkedListNode = exports.DoublyLinkedList = void 0;

var tsplus_module_1 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/structure/Hash"));

var tsplus_module_2 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/structure/Equals"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

class LinkedListNode {
  constructor(value) {
    this.value = value;
    this.removed = false;
    this.right = undefined;
    this.left = undefined;
  }

}
/**
 * @tsplus type DoublyLinkedList
 */


exports.LinkedListNode = LinkedListNode;

class DoublyLinkedList {
  constructor() {
    this.length = 0;
    this.headN = undefined;
    this.tailN = undefined;
  }

  get head() {
    return this.headN === undefined ? undefined : this.headN.value;
  }

  [tsplus_module_1.Hash.sym]() {
    return tsplus_module_1.hashRandomCached(this);
  }

  [tsplus_module_2.Equals.sym](that) {
    return this === that;
  }

  get isEmpty() {
    return this.length === 0;
  }

  get tail() {
    return this.tailN === undefined ? undefined : this.tailN.value;
  }

  forEach(f) {
    let current = this.headN;

    while (current !== undefined) {
      f(current.value);
      current = current.right;
    }
  }

  add(value) {
    const node = new LinkedListNode(value);

    if (this.length === 0) {
      this.headN = node;
    }

    if (this.tailN === undefined) {
      this.tailN = node;
    } else {
      this.tailN.right = node;
      node.left = this.tailN;
      this.tailN = node;
    }

    this.length += 1;
    return node;
  }

  empty() {
    this.length = 0;
    this.headN = this.tailN = undefined;
  }

  pop() {
    const h = this.tailN;

    if (h !== undefined) {
      this.remove(h);
      return h.value;
    }

    return undefined;
  }

  remove(n) {
    if (n.removed) {
      return;
    }

    n.removed = true;

    if (n.left !== undefined && n.right !== undefined) {
      n.left.right = n.right;
      n.right.left = n.left;
    } else if (n.left !== undefined) {
      this.tailN = n.left;
      n.left.right = undefined;
    } else if (n.right !== undefined) {
      this.headN = n.right;
      n.right.left = undefined;
    } else {
      this.tailN = undefined;
      this.headN = undefined;
    }

    if (this.length > 0) {
      this.length -= 1;
    }
  }

  shift() {
    const h = this.headN;

    if (h !== undefined) {
      this.remove(h);
      return h.value;
    }

    return undefined;
  }

  [Symbol.iterator]() {
    let done = false; // eslint-disable-next-line @typescript-eslint/no-this-alias

    let head = this.headN;
    return {
      next() {
        if (done) {
          return this.return();
        }

        if (head == null) {
          done = true;
          return this.return();
        }

        const value = head.value;
        head = head.right;
        return {
          done,
          value
        };
      },

      return(value) {
        if (!done) {
          done = true;
        }

        return {
          done: true,
          value
        };
      }

    };
  }

}

exports.DoublyLinkedList = DoublyLinkedList;
//# sourceMappingURL=DoublyLinkedList.js.map