"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ListBuffer = void 0;
exports.toCollection = toCollection;

var tsplus_module_1 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/List/definition"));

var tsplus_module_2 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/structure/Hash"));

var tsplus_module_3 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/structure/Equals"));

var tsplus_module_4 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/exceptions"));

var tsplus_module_5 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/List/reduce"));

var tsplus_module_6 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/List/unsafeTail"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

/*
 * This file is ported from
 *
 * Scala (https://www.scala-lang.org)
 *
 * Copyright EPFL and Lightbend, Inc.
 *
 * Licensed under Apache License 2.0
 * (http://www.apache.org/licenses/LICENSE-2.0).
 */

/**
 * @tsplus type ListBuffer
 * @tsplus companion ListBuffer.Ops
 */
class ListBuffer {
  constructor() {
    this.first = tsplus_module_1.nil();
    this.last0 = undefined;
    this.len = 0;
  }

  [Symbol.iterator]() {
    return this.first[Symbol.iterator]();
  }

  [tsplus_module_2.Hash.sym]() {
    return tsplus_module_2.hashRandomCached(this);
  }

  [tsplus_module_3.Equals.sym](that) {
    return this === that;
  }

  static empty() {
    return new ListBuffer();
  }

  static from(as) {
    const buf = new ListBuffer();

    for (const a of as) {
      buf.append(a);
    }

    return buf;
  }

  get length() {
    return this.len;
  }

  get isEmpty() {
    return this.len === 0;
  }

  get unsafeHead() {
    if (this.isEmpty) {
      return undefined;
    }

    return this.first.head;
  }

  get unsafeTail() {
    if (this.isEmpty) {
      return undefined;
    }

    return this.first.tail;
  }

  append(elem) {
    const last1 = tsplus_module_1.cons(elem, tsplus_module_1.nil());

    if (this.len === 0) {
      this.first = last1;
    } else {
      this.last0.tail = last1;
    }

    this.last0 = last1;
    this.len += 1;
    return this;
  }

  prepend(elem) {
    this.insert(0, elem);
    return this;
  }

  unprepend() {
    if (this.isEmpty) {
      throw new tsplus_module_4.NoSuchElement();
    }

    const h = this.first.head;
    this.first = this.first.tail;
    this.len -= 1;
    return h;
  }

  get toList() {
    return this.first;
  }

  insert(idx, elem) {
    if (idx < 0 || idx > this.len) {
      throw new tsplus_module_4.IndexOutOfBounds(idx, 0, this.len - 1);
    }

    if (idx === this.len) {
      this.append(elem);
    } else {
      const p = this.locate(idx);
      const nx = tsplus_module_1.cons(elem, this.getNext(p));

      if (p === undefined) {
        this.first = nx;
      } else {
        ;
        p.tail = nx;
      }

      this.len += 1;
    }

    return this;
  }

  reduce(b, f) {
    return tsplus_module_5.reduce(b, f)(this.first);
  }

  getNext(p) {
    if (p === undefined) {
      return this.first;
    } else {
      return tsplus_module_6.unsafeTail(p);
    }
  }

  locate(i) {
    if (i === 0) {
      return undefined;
    } else if (i === this.len) {
      return this.last0;
    } else {
      let p = this.first;

      for (let j = i - 1; j > 0; j--) {
        p = tsplus_module_6.unsafeTail(p);
      }

      return p;
    }
  }

}
/**
 * @tsplus getter ListBuffer toCollection
 */


exports.ListBuffer = ListBuffer;

function toCollection(self) {
  return self;
}
//# sourceMappingURL=ListBuffer.js.map