"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports._V = exports._K = exports.MutableHashMapSym = exports.MutableHashMapAspects = exports.MutableHashMap = void 0;

var tsplus_module_1 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/data/AtomicNumber"));

var tsplus_module_2 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/structure/Hash"));

var tsplus_module_3 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/structure/Equals"));

var tsplus_module_4 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/data/Maybe/definition"));

var tsplus_module_5 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/ImmutableArray/from"));

var tsplus_module_6 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/ImmutableArray/map"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

var _a;

const MutableHashMapSym = /*#__PURE__*/Symbol.for("@tsplus/stdlib/collections/mutable/MutableHashMap");
exports.MutableHashMapSym = MutableHashMapSym;

const _K = /*#__PURE__*/Symbol.for("@tsplus/stdlib/collections/mutable/MutableHashMap/K");

exports._K = _K;

const _V = /*#__PURE__*/Symbol.for("@tsplus/stdlib/collections/mutable/MutableHashMap/V");
/**
 * A mutable `HashMap`.
 *
 * @tsplus type MutableHashMap
 * @tsplus companion MutableHashMap.Ops
 */


exports._V = _V;

class MutableHashMap {
  constructor() {
    this[_a] = MutableHashMapSym;
    this.backingMap = new Map();
    this.length = new tsplus_module_1.AtomicNumber(0);
  }

  get size() {
    return this.length.get;
  }

  [(_a = MutableHashMapSym, tsplus_module_2.Hash.sym)]() {
    return tsplus_module_2.hashRandomCached(this);
  }

  [tsplus_module_3.Equals.sym](that) {
    return this === that;
  }

  get(k) {
    const hash = tsplus_module_2.hashUnknown(k);
    const arr = this.backingMap.get(hash);

    if (arr == null) {
      return tsplus_module_4.none;
    }

    let c = arr;

    while (c) {
      if (tsplus_module_3.equals(k, c.k)) {
        return tsplus_module_4.some(c.v);
      }

      c = c.next;
    }

    return tsplus_module_4.none;
  }

  remove(k) {
    const hash = tsplus_module_2.hashUnknown(k);
    const arr = this.backingMap.get(hash);

    if (arr == null) {
      return this;
    }

    if (tsplus_module_3.equals(k, arr.k)) {
      if (arr.next != null) {
        this.backingMap.set(hash, arr.next);
      } else {
        this.backingMap.delete(hash);
      }

      this.length.decrementAndGet();
      return this;
    }

    let next = arr.next;
    let curr = arr;

    while (next) {
      if (tsplus_module_3.equals(k, next.k)) {
        curr.next = next.next;
        this.length.decrementAndGet();
        return this;
      }

      curr = next;
      next = next.next;
    }

    return this;
  }

  set(k, v) {
    const hash = tsplus_module_2.hashUnknown(k);
    const arr = this.backingMap.get(hash);

    if (arr == null) {
      this.backingMap.set(hash, new Node(k, v));
      this.length.incrementAndGet();
      return this;
    }

    let c = arr;
    let l = arr;

    while (c) {
      if (tsplus_module_3.equals(k, c.k)) {
        c.v = v;
        return this;
      }

      l = c;
      c = c.next;
    }

    this.length.incrementAndGet();
    l.next = new Node(k, v);
    return this;
  }

  update(k, f) {
    const hash = tsplus_module_2.hashUnknown(k);
    const arr = this.backingMap.get(hash);

    if (arr == null) {
      return this;
    }

    let c = arr;

    while (c) {
      if (tsplus_module_3.equals(k, c.k)) {
        c.v = f(c.v);
        return this;
      }

      c = c.next;
    }

    return this;
  }

  [Symbol.iterator]() {
    return tsplus_module_6.map(node => [node.k, node.v])(tsplus_module_5.from(this.backingMap.values()))[Symbol.iterator]();
  }

}

exports.MutableHashMap = MutableHashMap;

class Node {
  constructor(k, v, next) {
    this.k = k;
    this.v = v;
    this.next = next;
  }

  [Symbol.iterator]() {
    // eslint-disable-next-line @typescript-eslint/no-this-alias
    let c = this;
    let n = 0;
    return {
      next: () => {
        if (c) {
          const kv = [c.k, c.v];
          c = c.next;
          n++;
          return {
            value: kv,
            done: false
          };
        } else {
          return {
            value: n,
            done: true
          };
        }
      }
    };
  }

}
/**
 * @tsplus static MutableHashMap.Ops $
 */


const MutableHashMapAspects = {};
exports.MutableHashMapAspects = MutableHashMapAspects;
//# sourceMappingURL=definition.js.map