"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.IterableWeakMap = void 0;
exports.getMaybe = getMaybe;
exports.make = make;
exports.toCollection = toCollection;

var tsplus_module_1 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/data/Maybe/definition"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

const IterableWeakMap = {
  $: {}
};
/**
 * @tsplus pipeable-index IterableWeakMap
 * @tsplus static IterableWeakMap.Aspects getMaybe
 * @tsplus pipeable IterableWeakMap getMaybe
 */

exports.IterableWeakMap = IterableWeakMap;

function getMaybe(key) {
  return self => self.has(key) ? tsplus_module_1.some(self.get(key)) : tsplus_module_1.none;
}
/**
 * @tsplus static IterableWeakMap.Ops make
 * @tsplus static IterableWeakMap.Ops __call
 */


function make(iterable) {
  return new ConcreteImpl(iterable);
}

class WeakImpl {
  static cleanup({
    ref,
    set
  }) {
    set.delete(ref);
  }

  constructor(iterable) {
    this.weakMap = new WeakMap();
    this.refSet = new Set();
    this.finalizationGroup = new FinalizationRegistry(WeakImpl.cleanup);

    for (const [key, value] of iterable) {
      this.set(key, value);
    }
  }

  set(key, value) {
    const ref = new WeakRef(key);
    this.weakMap.set(key, {
      value,
      ref
    });
    this.refSet.add(ref);
    this.finalizationGroup.register(key, {
      set: this.refSet,
      ref
    }, ref);
    return this;
  }

  get(key) {
    const entry = this.weakMap.get(key);
    return entry && entry.value;
  }

  delete(key) {
    const entry = this.weakMap.get(key);

    if (!entry) {
      return false;
    }

    this.weakMap.delete(key);
    this.refSet.delete(entry.ref);
    this.finalizationGroup.unregister(entry.ref);
    return true;
  }

  [Symbol.iterator]() {
    return this.entries();
  }

  *iterator() {
    for (const ref of this.refSet) {
      const key = ref.deref();
      if (!key) continue;
      const {
        value
      } = this.weakMap.get(key);
      yield [key, value];
    }
  }

  *entries() {
    for (const ref of this.refSet) {
      const key = ref.deref();
      if (!key) continue;
      const {
        value
      } = this.weakMap.get(key);
      yield [key, value];
    }
  }

  *keys() {
    for (const [key] of this) {
      yield key;
    }
  }

  *values() {
    for (const [, value] of this) {
      yield value;
    }
  }

  has(key) {
    return this.weakMap.has(key);
  }

}

const ConcreteImpl = typeof FinalizationRegistry !== "undefined" ? WeakImpl : Map;
/**
 * @tsplus getter IterableWeakMap toCollection
 */

function toCollection(self) {
  return self;
}
//# sourceMappingURL=IterableWeakMap.js.map