export declare const OrPatchSym: unique symbol;
export type OrPatchSym = typeof OrPatchSym;
export declare const OrPatchValueSym: unique symbol;
export type OrPatchValueSym = typeof OrPatchValueSym;
export declare const OrPatchValue2Sym: unique symbol;
export type OrPatchValue2Sym = typeof OrPatchValueSym;
export declare const OrPatchPatchSym: unique symbol;
export type OrPatchPatchSym = typeof OrPatchPatchSym;
export declare const OrPatchPatch2Sym: unique symbol;
export type OrPatchPatch2Sym = typeof OrPatchPatch2Sym;
/**
 * A patch which describes updates to either one value or another.
 *
 * @tsplus type Differ.Or.Patch
 */
export interface OrPatch<in out Value, in out Value2, in out Patch, in out Patch2> {
    readonly [OrPatchSym]: OrPatchSym;
    readonly [OrPatchValueSym]: (_: Value) => Value;
    readonly [OrPatchValue2Sym]: (_: Value2) => Value2;
    readonly [OrPatchPatchSym]: (_: Patch) => Patch;
    readonly [OrPatchPatch2Sym]: (_: Patch2) => Patch2;
}
/**
 * @tsplus type Differ.Or.Patch.Ops
 */
export interface OrPatchOps {
    readonly $: OrPatchAspects;
}
/**
 * @tsplus static Differ.Ops Or
 * @tsplus location "@tsplus/stdlib/data/Differ/OrPatch/definition"
 */
export declare const OrPatch: OrPatchOps;
/**
 * @tsplus type Differ.Or.Patch.Aspects
 */
export interface OrPatchAspects {
}
/**
 * @tsplus unify Differ.Or.Patch
 */
export declare function unifyOrPatch<X extends OrPatch<any, any, any, any>>(self: X): OrPatch<[
    X
] extends [{
    [OrPatchValueSym]: (_: infer Value) => infer Value;
}] ? Value : never, [
    X
] extends [{
    [OrPatchValue2Sym]: (_: infer Value2) => infer Value2;
}] ? Value2 : never, [
    X
] extends [{
    [OrPatchPatchSym]: (_: infer Patch) => infer Patch;
}] ? Patch : never, [
    X
] extends [{
    [OrPatchPatch2Sym]: (_: infer Patch2) => infer Patch2;
}] ? Patch2 : never>;
export declare abstract class BaseOrPatch<Value, Value2, Patch, Patch2> implements OrPatch<Value, Value2, Patch, Patch2> {
    readonly [OrPatchSym]: OrPatchSym;
    readonly [OrPatchValueSym]: (_: Value) => Value;
    readonly [OrPatchValue2Sym]: (_: Value2) => Value2;
    readonly [OrPatchPatchSym]: (_: Patch) => Patch;
    readonly [OrPatchPatch2Sym]: (_: Patch2) => Patch2;
}
export declare class AndThenOrPatch<Value, Value2, Patch, Patch2> extends BaseOrPatch<Value, Value2, Patch, Patch2> {
    readonly first: OrPatch<Value, Value2, Patch, Patch2>;
    readonly second: OrPatch<Value, Value2, Patch, Patch2>;
    readonly _tag = "AndThen";
    constructor(first: OrPatch<Value, Value2, Patch, Patch2>, second: OrPatch<Value, Value2, Patch, Patch2>);
}
export declare class EmptyOrPatch<Value, Value2, Patch, Patch2> extends BaseOrPatch<Value, Value2, Patch, Patch2> {
    readonly _tag = "Empty";
}
export declare class SetLeftOrPatch<Value, Value2, Patch, Patch2> extends BaseOrPatch<Value, Value2, Patch, Patch2> {
    readonly value: Value;
    readonly _tag = "SetLeft";
    constructor(value: Value);
}
export declare class SetRightOrPatch<Value, Value2, Patch, Patch2> extends BaseOrPatch<Value, Value2, Patch, Patch2> {
    readonly value: Value2;
    readonly _tag = "SetRight";
    constructor(value: Value2);
}
export declare class UpdateLeftOrPatch<Value, Value2, Patch, Patch2> extends BaseOrPatch<Value, Value2, Patch, Patch2> {
    readonly patch: Patch;
    readonly _tag = "UpdateLeft";
    constructor(patch: Patch);
}
export declare class UpdateRightOrPatch<Value, Value2, Patch, Patch2> extends BaseOrPatch<Value, Value2, Patch, Patch2> {
    readonly patch: Patch2;
    readonly _tag = "UpdateRight";
    constructor(patch: Patch2);
}
export type OrPatchInstruction = AndThenOrPatch<any, any, any, any> | EmptyOrPatch<any, any, any, any> | SetLeftOrPatch<any, any, any, any> | SetRightOrPatch<any, any, any, any> | UpdateLeftOrPatch<any, any, any, any> | UpdateRightOrPatch<any, any, any, any>;
/**
 * @tsplus macro identity
 */
export declare function orPatchInstruction<Value, Value2, Patch, Patch2>(self: OrPatch<Value, Value2, Patch, Patch2>): OrPatchInstruction;
//# sourceMappingURL=definition.d.ts.map