"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.apply = apply;

var tsplus_module_1 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/data/Differ/definition"));

var tsplus_module_2 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/List/from"));

var tsplus_module_3 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/List/definition"));

var tsplus_module_4 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/List/prepend"));

var tsplus_module_5 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/data/Either/definition"));

var _definition4 = /*#__PURE__*/require("@tsplus/stdlib/data/Differ/OrPatch/definition");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

/**
 * Applies an or patch to a value to produce a new value which represents
 * the original value updated with the changes described by this patch.
 *
 * @tsplus static Differ.Or.Patch.Aspects apply
 * @tsplus pipeable Differ.Or.Patch apply
 */
function apply(oldValue, left, right) {
  return self => applyLoop(left, right, oldValue, tsplus_module_2.make(self));
}
/**
 * @tsplus tailRec
 */


function applyLoop(left, right, either, patches) {
  var left_1 = left,
      right_1 = right,
      either_1 = either,
      patches_1 = patches;
  var left_2 = left,
      right_2 = right,
      either_2 = either,
      patches_2 = patches;

  while (1) {
    if (tsplus_module_3.isNil(patches_1)) {
      return either_1;
    }

    const patch = patches_1.head;
    const nextPatches = patches_1.tail;

    switch (patch._tag) {
      case "AndThen":
        {
          left_2 = left_1;
          right_2 = right_1;
          either_2 = either_1;
          patches_2 = tsplus_module_4.prepend(patch.first)(tsplus_module_4.prepend(patch.second)(nextPatches));
          left_1 = left_2;
          right_1 = right_2;
          either_1 = either_2;
          patches_1 = patches_2;
          continue;
        }

      case "Empty":
        {
          left_2 = left_1;
          right_2 = right_1;
          either_2 = either_1;
          patches_2 = nextPatches;
          left_1 = left_2;
          right_1 = right_2;
          either_1 = either_2;
          patches_1 = patches_2;
          continue;
        }

      case "UpdateLeft":
        {
          switch (either_1._tag) {
            case "Left":
              {
                left_2 = left_1;
                right_2 = right_1;
                either_2 = tsplus_module_5.left(left_1.patch(patch.patch, either_1.left));
                patches_2 = nextPatches;
                left_1 = left_2;
                right_1 = right_2;
                either_1 = either_2;
                patches_1 = patches_2;
                continue;
              }

            case "Right":
              {
                left_2 = left_1;
                right_2 = right_1;
                either_2 = either_1;
                patches_2 = nextPatches;
                left_1 = left_2;
                right_1 = right_2;
                either_1 = either_2;
                patches_1 = patches_2;
                continue;
              }
          }
        }

      case "UpdateRight":
        {
          switch (either_1._tag) {
            case "Left":
              {
                left_2 = left_1;
                right_2 = right_1;
                either_2 = either_1;
                patches_2 = nextPatches;
                left_1 = left_2;
                right_1 = right_2;
                either_1 = either_2;
                patches_1 = patches_2;
                continue;
              }

            case "Right":
              {
                left_2 = left_1;
                right_2 = right_1;
                either_2 = tsplus_module_5.right(right_1.patch(patch.patch, either_1.right));
                patches_2 = nextPatches;
                left_1 = left_2;
                right_1 = right_2;
                either_1 = either_2;
                patches_1 = patches_2;
                continue;
              }
          }
        }

      case "SetLeft":
        {
          left_2 = left_1;
          right_2 = right_1;
          either_2 = tsplus_module_5.left(patch.value);
          patches_2 = nextPatches;
          left_1 = left_2;
          right_1 = right_2;
          either_1 = either_2;
          patches_1 = patches_2;
          continue;
        }

      case "SetRight":
        {
          left_2 = left_1;
          right_2 = right_1;
          either_2 = tsplus_module_5.right(patch.value);
          patches_2 = nextPatches;
          left_1 = left_2;
          right_1 = right_2;
          either_1 = either_2;
          patches_1 = patches_2;
          continue;
        }
    }
  }
}
//# sourceMappingURL=apply.js.map