import { Equals } from "@tsplus/stdlib/structure/Equals";
import { Hash } from "@tsplus/stdlib/structure/Hash";
import { IsInt } from "@tsplus/stdlib/utilities/Types";
export declare const DurationSym: unique symbol;
export type DurationSym = typeof DurationSym;
/**
 * @tsplus type Duration
 */
export interface Duration extends Equals {
    readonly [DurationSym]: DurationSym;
    readonly millis: number;
}
export declare class DurationInternal implements Duration {
    readonly millis: number;
    readonly [DurationSym]: DurationSym;
    constructor(millis: number);
    [Hash.sym](this: this): number;
    [Equals.sym](this: this, other: unknown): boolean;
}
/**
 * @tsplus static Duration.Ops isDuration
 * @tsplus location "@tsplus/stdlib/data/Duration"
 */
export declare function isDuration(u: unknown): u is Duration;
/**
 * @tsplus type Duration.Ops
 */
export interface DurationOps {
    readonly $: DurationAspects;
    readonly Zero: Duration;
    readonly Infinity: Duration;
}
export declare const Duration: DurationOps;
/**
 * @tsplus type Duration.Aspects
 */
export interface DurationAspects {
}
/**
 * @tsplus getter number millis
 * @tsplus static Duration.Ops millis
 * @tsplus location "@tsplus/stdlib/data/Duration"
 */
export declare function millis<N extends number>(self: IsInt<N>): DurationInternal;
/**
 * @tsplus getter number seconds
 * @tsplus static Duration.Ops seconds
 * @tsplus location "@tsplus/stdlib/data/Duration"
 */
export declare function seconds<N extends number>(self: IsInt<N>): DurationInternal;
/**
 * @tsplus getter number minutes
 * @tsplus static Duration.Ops minutes
 * @tsplus location "@tsplus/stdlib/data/Duration"
 */
export declare function minutes<N extends number>(self: IsInt<N>): DurationInternal;
/**
 * @tsplus getter number hours
 * @tsplus static Duration.Ops hours
 * @tsplus location "@tsplus/stdlib/data/Duration"
 */
export declare function hours<N extends number>(self: IsInt<N>): DurationInternal;
/**
 * @tsplus getter number days
 * @tsplus static Duration.Ops days
 * @tsplus location "@tsplus/stdlib/data/Duration"
 */
export declare function days<N extends number>(self: IsInt<N>): DurationInternal;
/**
 * @tsplus getter number weeks
 * @tsplus static Duration.Ops weeks
 * @tsplus location "@tsplus/stdlib/data/Duration"
 */
export declare function weeks<N extends number>(self: IsInt<N>): DurationInternal;
/**
 * @tsplus pipeable-operator Duration *
 * @tsplus static Duration.Aspects times
 * @tsplus pipeable Duration times
 * @tsplus location "@tsplus/stdlib/data/Duration"
 */
export declare function times<N extends number>(times: IsInt<N>): (self: Duration) => Duration;
/**
 * @tsplus operator Duration *
 * @tsplus location "@tsplus/stdlib/data/Duration"
 */
export declare function timesInverted<N extends number>(times: IsInt<N>, self: Duration): DurationInternal;
/**
 * @tsplus pipeable-operator Duration +
 * @tsplus static Duration.Aspects add
 * @tsplus pipeable Duration add
 * @tsplus location "@tsplus/stdlib/data/Duration"
 */
export declare function add(that: Duration): (self: Duration) => Duration;
/**
 * @tsplus pipeable-operator Duration -
 * @tsplus static Duration.Aspects subtract
 * @tsplus pipeable Duration subtract
 * @tsplus location "@tsplus/stdlib/data/Duration"
 */
export declare function subtract(that: Duration): (self: Duration) => Duration;
/**
 * @tsplus pipeable-operator Duration <
 * @tsplus static Duration.Aspects lowerThen
 * @tsplus pipeable Duration lowerThen
 * @tsplus location "@tsplus/stdlib/data/Duration"
 */
export declare function lowerThen(that: Duration): (self: Duration) => boolean;
/**
 * @tsplus pipeable-operator Duration <=
 * @tsplus static Duration.Aspects lowerThenOrEqual
 * @tsplus pipeable Duration lowerThenOrEqual
 * @tsplus location "@tsplus/stdlib/data/Duration"
 */
export declare function lowerThenOrEqual(that: Duration): (self: Duration) => boolean;
/**
 * @tsplus pipeable-operator Duration >
 * @tsplus static Duration.Aspects greaterThen
 * @tsplus pipeable Duration greaterThen
 * @tsplus location "@tsplus/stdlib/data/Duration"
 */
export declare function greaterThen(that: Duration): (self: Duration) => boolean;
/**
 * @tsplus pipeable-operator Duration >=
 * @tsplus static Duration.Aspects greaterThenOrEqual
 * @tsplus pipeable Duration greaterThenOrEqual
 * @tsplus location "@tsplus/stdlib/data/Duration"
 */
export declare function greaterThenOrEqual(that: Duration): (self: Duration) => boolean;
/**
 * @tsplus pipeable-operator Duration ==
 * @tsplus static Duration.Aspects equals
 * @tsplus pipeable Duration equals
 * @tsplus location "@tsplus/stdlib/data/Duration"
 */
export declare function equals(that: Duration): (self: Duration) => boolean;
//# sourceMappingURL=Duration.d.ts.map