"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Either = void 0;
exports.equals = equals;
exports.getLeft = getLeft;
exports.getRight = getRight;
exports.isEither = isEither;
exports.isLeft = isLeft;
exports.isRight = isRight;
exports.left = left;
exports.leftW = leftW;
exports.right = right;
exports.rightW = rightW;
exports.unifyEither = unifyEither;
exports.widenA = widenA;
exports.widenA_ = widenA_;
exports.widenE = widenE;
exports.widenE_ = widenE_;

var tsplus_module_1 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/data/Maybe/definition"));

var tsplus_module_2 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/structure/Equals"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

const Either = {
  $: {}
};
/**
 * @tsplus unify Either
 * @tsplus unify Either.Left
 * @tsplus unify Either.Right
 */

exports.Either = Either;

function unifyEither(self) {
  return self;
}
/**
 * Returns `true` if the `Either` is an instance of `Left`, `false` otherwise.
 *
 * @tsplus fluent Either isLeft
 */


function isLeft(ma) {
  return ma._tag === "Left";
}
/**
 * Returns `true` if the `Either` is an instance of `Right`, `false` otherwise.
 *
 * @tsplus fluent Either isRight
 */


function isRight(ma) {
  return ma._tag === "Right";
}
/**
 * @tsplus getter Either left
 */


function getLeft(self) {
  return self._tag === "Left" ? tsplus_module_1.some(self.left) : tsplus_module_1.none;
}
/**
 * @tsplus static Either.Ops isEither
 */


function isEither(u) {
  return typeof u === "object" && u != null && "_tag" in u && (u["_tag"] === "Left" || u["_tag"] === "Right");
}
/**
 * @tsplus getter Either right
 */


function getRight(self) {
  return self._tag === "Right" ? tsplus_module_1.some(self.right) : tsplus_module_1.none;
}
/**
 * Constructs a new `Either` holding a `Right` value. This usually represents a
 * successful value due to the right bias of this structure.
 *
 * @tsplus static Either.Ops right
 * @tsplus static Either.Ops __call
 */


function right(a) {
  return {
    _tag: "Right",
    right: a
  };
}
/**
 * Constructs a new `Either` holding a `Right` value. This usually represents a
 * successful value due to the right bias of this structure.
 *
 * @tsplus static Either.Ops rightW
 */


function rightW(a) {
  return {
    _tag: "Right",
    right: a
  };
}
/**
 * Constructs a new `Either` holding a `Left` value. This usually represents a
 * failure, due to the right-bias of this structure.
 *
 * @tsplus static Either.Ops left
 */


function left(e) {
  return {
    _tag: "Left",
    left: e
  };
}
/**
 * Constructs a new `Either` holding a `Left` value. This usually represents a
 * failure, due to the right-bias of this structure.
 *
 * @tsplus static Either.Ops leftW
 */


function leftW(e) {
  return {
    _tag: "Left",
    left: e
  };
}
/**
 * Widen left side `Either[E, A] => Either[E | E1, A]`.
 *
 * @tsplus getter Either widenE
 * @tsplus macro identity
 */


function widenE_(self) {
  return self;
}
/**
 * Widen left side `Either[E, A] => Either[E | E1, A]`.
 *
 * @tsplus static Either.Aspects widenE
 */


function widenE() {
  return (
    /**
     * @tsplus macro identity
     */
    self => self
  );
}
/**
 * Widen left side `Either[E, A] => Either[E | E1, A]`.
 *
 * @tsplus getter Either widenA
 * @tsplus macro identity
 */


function widenA_(self) {
  return self;
}
/**
 * Widen right side `Either[E, A] => Either[E, A | A1]`.
 *
 * @tsplus static Either.Aspects widenA
 */


function widenA() {
  return (
    /**
     * @tsplus macro identity
     */
    self => self
  );
}
/**
 * @tsplus operator Either ==
 */


function equals(a, b) {
  return tsplus_module_2.equals(a, b);
}
//# sourceMappingURL=definition.js.map