declare global {
    /**
     * @tsplus type string
     */
    export interface String {
    }
    /**
     * @tsplus type string.Ops
     */
    export interface StringConstructor {
    }
}
/**
 * Keep the specified number of characters from the start of a string.
 *
 * If `n` is larger than the available number of characters, the string will
 * be returned whole.
 *
 * If `n` is not a positive number, an empty string will be returned.
 *
 * If `n` is a float, it will be rounded down to the nearest integer.
 * @tsplus pipeable string takeLeft
 * @tsplus location "@tsplus/stdlib/data/String"
 */
export declare function takeLeft(n: number): (self: string) => string;
/**
 * Keep the specified number of characters from the end of a string.
 *
 * If `n` is larger than the available number of characters, the string will
 * be returned whole.
 *
 * If `n` is not a positive number, an empty string will be returned.
 *
 * If `n` is a float, it will be rounded down to the nearest integer.
 * @tsplus pipeable string takeRight
 * @tsplus location "@tsplus/stdlib/data/String"
 */
export declare function takeRight(n: number): (s: string) => string;
/**
 * Represents the character code of a carriage return character (`"\r"`).
 * @tsplus static string.Ops CR
 * @tsplus location "@tsplus/stdlib/data/String"
 */
export declare const CR = 13;
/**
 * Represents the character code of a line-feed character (`"\n"`).
 * @tsplus static string.Ops LF
 * @tsplus location "@tsplus/stdlib/data/String"
 */
export declare const LF = 10;
/**
 * Returns an `IterableIterator` which yields each line contained within the
 * string, trimming off the trailing newline character.
 * @tsplus getter string linesIterator
 * @tsplus location "@tsplus/stdlib/data/String"
 */
export declare function linesIterator(self: string): LinesIterator;
/**
 * Returns an `IterableIterator` which yields each line contained within the
 * string as well as the trailing newline character.
 * @tsplus getter string linesIterator
 * @tsplus location "@tsplus/stdlib/data/String"
 */
export declare function linesWithSeparators(s: string): LinesIterator;
/**
 * For every line in this string, strip a leading prefix consisting of blanks
 * or control characters followed by the character specified by `marginChar`
 * from the line.
 * @tsplus pipeable string stripMarginWith
 * @tsplus location "@tsplus/stdlib/data/String"
 */
export declare function stripMarginWith(marginChar: string): (self: string) => string;
/**
 * For every line in this string, strip a leading prefix consisting of blanks
 * or control characters followed by the `"|"` character from the line.
 * @tsplus getter string stripMargin
 * @tsplus location "@tsplus/stdlib/data/String"
 */
export declare function stripMargin(self: string): string;
export declare class LinesIterator implements IterableIterator<string> {
    readonly s: string;
    readonly stripped: boolean;
    private index;
    private length;
    constructor(s: string, stripped?: boolean);
    next(): IteratorResult<string>;
    [Symbol.iterator](): IterableIterator<string>;
    private get done();
}
//# sourceMappingURL=String.d.ts.map