/**
 * An ArrayInt represents an integer larger than what can be represented in
 * classical JavaScript.
 *
 * The values stored in data must be in the range `[0, 0xffffffff]`.
 *
 * ```ts
 * { sign:  1, data: [ 42 ] } // = 42
 * { sign: -1, data: [ 42 ] } // = -42
 * { sign: -1, data: [ 5, 42 ] } // = -1 * (5 * 2**32 + 42)
 * { sign: -1, data: [ 1, 5, 42 ] } // = -1 * (1 * 2**64 + 5 * 2**32 + 42)
 * ```
 */
export interface ArrayInt {
    /**
     * Sign of the represented number
     */
    sign: -1 | 1;
    /**
     * Value of the number, must only contain numbers in the range [0, 0xffffffff]
     */
    data: number[];
}
/**
 * Trim uneeded zeros in ArrayInt
 * and uniform notation for zero: {sign: 1, data: [0]}
 */
export declare function trimArrayIntInplace(arrayInt: ArrayInt): ArrayInt;
//# sourceMappingURL=ArrayInt.d.ts.map