"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.uniformArrayIntDistributionInternal = uniformArrayIntDistributionInternal;

var _uniformIntDistributionInternal = /*#__PURE__*/require("@tsplus/stdlib/io/Random/Distribution/_internal/uniformIntDistributionInternal");

/**
 * Uniformly generate an `ArrayInt` in the range [0, rangeSize].
 *
 * @remarks
 * In the worst case scenario it may discard half of the randomly generated
 * values. Worst case being: most significant number is `1` and remaining part
 * evaluates to `0`.
 *
 * @internal
 */
function uniformArrayIntDistributionInternal(out, rangeSize, rng) {
  const rangeLength = rangeSize.length; // We iterate until we find a valid value for arrayInt
  // eslint-disable-next-line no-constant-condition

  while (true) {
    // We compute a new value for arrayInt
    for (let index = 0; index !== rangeLength; ++index) {
      const indexRangeSize = index === 0 ? rangeSize[0] + 1 : 0x100000000;
      out[index] = (0, _uniformIntDistributionInternal.uniformIntDistributionInternal)(indexRangeSize, rng);
    } // If in the correct range we can return it


    for (let index = 0; index !== rangeLength; ++index) {
      const current = out[index];
      const currentInRange = rangeSize[index];

      if (current < currentInRange) {
        return out; // arrayInt < rangeSize
      } else if (current > currentInRange) {
        break; // arrayInt > rangeSize
      }
    } // Otherwise we need to try another one

  }
}
//# sourceMappingURL=uniformArrayIntDistributionInternal.js.map