"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.uniformBigIntDistribution = uniformBigIntDistribution;

function uniformBigIntInternal(from, diff, rng) {
  const MinRng = BigInt(rng.min());
  const NumValues = BigInt(rng.max() - rng.min() + 1); // Number of iterations required to have enough random
  // to build uniform entries in the asked range

  let FinalNumValues = NumValues;
  let NumIterations = BigInt(1);

  while (FinalNumValues < diff) {
    FinalNumValues *= NumValues;
    ++NumIterations;
  }

  const MaxAcceptedRandom = FinalNumValues - FinalNumValues % diff; // eslint-disable-next-line no-constant-condition

  while (true) {
    // Aggregate mutiple calls to next() into a single random value
    let value = BigInt(0);

    for (let num = BigInt(0); num !== NumIterations; ++num) {
      const out = rng.next();
      value = NumValues * value + (BigInt(out) - MinRng);
    }

    if (value < MaxAcceptedRandom) {
      const inDiff = value % diff;
      return inDiff + from;
    }
  }
}

function uniformBigIntDistribution(from, to, rng) {
  const diff = to - from + BigInt(1);

  if (rng != null) {
    return uniformBigIntInternal(from, diff, rng);
  }

  return rng => uniformBigIntInternal(from, diff, rng);
}
//# sourceMappingURL=uniformBigIntDistribution.js.map