"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.MersenneTwister = void 0;
const N = 624;
const M = 397;
const UPPER_MASK = 0x80000000;
const LOWER_MASK = 0x7fffffff;
const MATRIX_A = 0x9908b0df;

class MersenneTwister {
  constructor(seed) {
    this.mt = new Array(N);
    this.mti = N + 1;
    this.setSeed(seed ?? Date.now());
  }

  setSeed(seed) {
    let s;
    this.mt[0] = seed >>> 0;

    for (this.mti = 1; this.mti < N; this.mti++) {
      s = this.mt[this.mti - 1] ^ this.mt[this.mti - 1] >>> 30;
      this.mt[this.mti] = (((s & 0xffff0000) >>> 16) * 1812433253 << 16) + (s & 0x0000ffff) * 1812433253 + this.mti;
      this.mt[this.mti] >>>= 0;
    }
  }

  seedArray(vector) {
    let i = 1,
        j = 0,
        k = N > vector.length ? N : vector.length,
        s;
    this.setSeed(19650218);

    for (; k > 0; k--) {
      s = this.mt[i - 1] ^ this.mt[i - 1] >>> 30;
      this.mt[i] = (this.mt[i] ^ (((s & 0xffff0000) >>> 16) * 1664525 << 16) + (s & 0x0000ffff) * 1664525) + vector[j] + j;
      this.mt[i] >>>= 0;
      i++;
      j++;

      if (i >= N) {
        this.mt[0] = this.mt[N - 1];
        i = 1;
      }

      if (j >= vector.length) {
        j = 0;
      }
    }

    for (k = N - 1; k; k--) {
      s = this.mt[i - 1] ^ this.mt[i - 1] >>> 30;
      this.mt[i] = (this.mt[i] ^ (((s & 0xffff0000) >>> 16) * 1566083941 << 16) + (s & 0x0000ffff) * 1566083941) - i;
      this.mt[i] >>>= 0;
      i++;

      if (i >= N) {
        this.mt[0] = this.mt[N - 1];
        i = 1;
      }
    }

    this.mt[0] = 0x80000000;
  }

  next() {
    let y,
        kk,
        // eslint-disable-next-line prefer-const
    mag01 = [0, MATRIX_A];

    if (this.mti >= N) {
      if (this.mti === N + 1) {
        this.setSeed(5489);
      }

      for (kk = 0; kk < N - M; kk++) {
        y = this.mt[kk] & UPPER_MASK | this.mt[kk + 1] & LOWER_MASK;
        this.mt[kk] = this.mt[kk + M] ^ y >>> 1 ^ mag01[y & 1];
      }

      for (; kk < N - 1; kk++) {
        y = this.mt[kk] & UPPER_MASK | this.mt[kk + 1] & LOWER_MASK;
        this.mt[kk] = this.mt[kk + (M - N)] ^ y >>> 1 ^ mag01[y & 1];
      }

      y = this.mt[N - 1] & UPPER_MASK | this.mt[0] & LOWER_MASK;
      this.mt[N - 1] = this.mt[M - 1] ^ y >>> 1 ^ mag01[y & 1];
      this.mti = 0;
    }

    y = this.mt[this.mti++];
    y ^= y >>> 11;
    y ^= y << 7 & 0x9d2c5680;
    y ^= y << 15 & 0xefc60000;
    y ^= y >>> 18;
    return y >>> 0;
  }

  min() {
    return 0;
  }

  max() {
    return 0xffffffff;
  }

}

exports.MersenneTwister = MersenneTwister;
//# sourceMappingURL=MersenneTwister.js.map