import type { RandomGenerator } from "@tsplus/stdlib/io/Random/Generator/RandomGenerator";
/**
 * Wrapper around an instance of a `RandomGenerator` random number generator
 * offering a simpler interface to deal with random with impure patterns.
 */
export declare class MutableRandom {
    private internalRng;
    private static MIN_INT;
    private static MAX_INT;
    private static DBL_FACTOR;
    private static DBL_DIVISOR;
    /**
     * Create a mutable random number generator.
     *
     * @param internalRng - Immutable random number generator
     */
    constructor(internalRng: RandomGenerator);
    /**
     * Clone the random number generator.
     */
    clone(): MutableRandom;
    /**
     * Generate an integer having `bits` random bits.
     *
     * @param bits - Number of bits to generate
     */
    next(bits: number): number;
    /**
     * Generate a random boolean.
     */
    nextBoolean(): boolean;
    /**
     * Generate a random integer (32 bits).
     */
    nextInt(): number;
    /**
     * Generate a random integer between min (inclusive) and max (inclusive).
     *
     * @param min - Minimal integer value
     * @param max - Maximal integer value
     */
    nextInt(min: number, max: number): number;
    /**
     * Generate a random `bigint` between min (inclusive) and max (inclusive).
     *
     * @param min - Minimal `bigint` value
     * @param max - Maximal `bigint` value
     */
    nextBigInt(min: bigint, max: bigint): bigint;
    /**
     * Generate a random `ArrayInt` between min (inclusive) and max (inclusive).
     *
     * @param min - Minimal `ArrayInt` value
     * @param max - Maximal `ArrayInt` value
     */
    nextArrayInt(min: {
        sign: 1 | -1;
        data: number[];
    }, max: {
        sign: 1 | -1;
        data: number[];
    }): {
        sign: 1 | -1;
        data: number[];
    };
    /**
     * Generate a random floating point number between 0.0 (inclusive) and 1.0
     * (exclusive).
     */
    nextDouble(): number;
    setSeed(seed: number): void;
    private uniformIn;
}
//# sourceMappingURL=MutableRandom.d.ts.map