import { HKT } from "@tsplus/stdlib/prelude/HKT";
import { LazyArg } from "@tsplus/stdlib/data/Function";
import { Monad } from "@tsplus/stdlib/prelude/Monad";
import { List } from "@tsplus/stdlib/collections/List/definition";
export declare class GenHKT<T, A> {
    readonly effect: T;
    constructor(effect: T);
    [Symbol.iterator](): Generator<GenHKT<T, A>, A, any>;
}
export declare class GenLazyHKT<T, A> {
    readonly effect: () => T;
    constructor(effect: () => T);
    [Symbol.iterator](): Generator<GenLazyHKT<T, A>, A, any>;
}
/**
 * To be used with multi-shot monads, required adapter to be lazy and is O(n^2)
 * perf wise because the generator needs to be replayed.
 * @tsplus static DSL genWithHistoryF
 * @tsplus location "@tsplus/stdlib/prelude/DSL/genF"
 */
export declare function genWithHistoryF<F extends HKT, ADAPTER = {
    <R, E, A>(_: LazyArg<HKT.Kind<F, R, E, A>>): GenLazyHKT<HKT.Kind<F, R, E, A>, A>;
}>(F: Monad<F>, config?: {
    adapter?: ADAPTER;
}): <Eff extends GenLazyHKT<import("../HKT").HKT.Kind<F, any, any, any>, any>, AEff>(f: (i: ADAPTER) => Generator<Eff, AEff, any>) => import("../HKT").HKT.Kind<F, import("../HKT").HKT.Infer<F, "R", ReturnType<Eff["effect"]>>, import("../HKT").HKT.Infer<F, "E", ReturnType<Eff["effect"]>>, AEff>;
/**
 * To be used in one-shot monads, adapter is eager and perf is native.
 * @tsplus static DSL genF
 * @tsplus location "@tsplus/stdlib/prelude/DSL/genF"
 */
export declare function genF<F extends HKT, ADAPTER = {
    <R, E, A>(_: HKT.Kind<F, R, E, A>): GenHKT<HKT.Kind<F, R, E, A>, A>;
}>(F: Monad<F>, config?: {
    adapter?: ADAPTER;
}): <Eff extends GenHKT<import("../HKT").HKT.Kind<F, any, any, any>, any>, AEff>(f: (i: ADAPTER) => Generator<Eff, AEff, any>) => import("../HKT").HKT.Kind<F, import("../HKT").HKT.Infer<F, "R", Eff["effect"]>, import("../HKT").HKT.Infer<F, "E", Eff["effect"]>, AEff>;
//# sourceMappingURL=genF.d.ts.map