"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.GenLazyHKT = exports.GenHKT = void 0;
exports.genF = genF;
exports.genWithHistoryF = genWithHistoryF;

var tsplus_module_1 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/prelude/DSL/flatMapF"));

var tsplus_module_2 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/prelude/DSL/succeedF"));

var tsplus_module_3 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/List/reverse"));

var tsplus_module_4 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/exceptions"));

var tsplus_module_5 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/List/prepend"));

var tsplus_module_6 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/List/empty"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

class GenHKT {
  constructor(effect) {
    this.effect = effect;
  }

  *[Symbol.iterator]() {
    return yield this;
  }

}

exports.GenHKT = GenHKT;

class GenLazyHKT {
  constructor(effect) {
    this.effect = effect;
  }

  *[Symbol.iterator]() {
    return yield this;
  }

}

exports.GenLazyHKT = GenLazyHKT;

const adapter = _ => {
  return new GenHKT(_);
};

const adapterLazy = _ => {
  return new GenHKT(_);
};
/**
 * To be used with multi-shot monads, required adapter to be lazy and is O(n^2)
 * perf wise because the generator needs to be replayed.
 *
 * @tsplus static DSL genWithHistoryF
 */


function genWithHistoryF(F, config) {
  return f => {
    const flatMap_ = tsplus_module_1.flatMapF_(F);
    const succeed = tsplus_module_2.succeedF(F);
    return flatMap_(succeed({}), () => {
      function run(replayStack) {
        const iterator = f(config?.adapter ? config.adapter : adapterLazy);
        let state = iterator.next();

        for (const a of tsplus_module_3.reverse(replayStack)) {
          if (state.done) {
            throw new tsplus_module_4.PrematureGeneratorExit();
          }

          state = iterator.next(a);
        }

        if (state.done) {
          return succeed(state.value);
        }

        return flatMap_(state.value["effect"](), val => {
          return run(tsplus_module_5.prepend(val)(replayStack));
        });
      }

      return run(tsplus_module_6.empty());
    });
  };
}
/**
 * To be used in one-shot monads, adapter is eager and perf is native.
 *
 * @tsplus static DSL genF
 */


function genF(F, config) {
  return f => {
    const flatMap_ = tsplus_module_1.flatMapF_(F);
    const succeed = tsplus_module_2.succeedF(F);
    return flatMap_(succeed({}), () => {
      const iterator = f(config?.adapter ? config.adapter : adapter);
      const state = iterator.next();

      function run(state) {
        if (state.done) {
          return succeed(state.value);
        }

        return flatMap_(state.value["effect"], val => {
          const next = iterator.next(val);
          return run(next);
        });
      }

      return run(state);
    });
  };
}
//# sourceMappingURL=genF.js.map