"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.between = between;
exports.clamp = clamp;
exports.contramap = contramap;
exports.getEquivalence = getEquivalence;
exports.gt = void 0;
exports.inverted = inverted;
exports.leq = leq;
exports.min = exports.max = exports.lt = void 0;

var tsplus_module_1 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/prelude/Ord/definition"));

var tsplus_module_2 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/prelude/Equivalence/definition"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

const gt = gt_1;
exports.gt = gt;
const lt = lt_1;
exports.lt = lt;
const max = max_1;
exports.max = max;
const min = min_1;
/**
 * Contramap Ord input
 *
 * @tsplus fluent Ord contramap
 */

exports.min = min;

function contramap(fa, f) {
  return tsplus_module_1.Ord((x, y) => fa.compare(f(x), f(y)));
}
/**
 * Test whether one value is _strictly greater than_ another
 *
 * @tsplus getter Ord gt
 */


function gt_1(O) {
  return (x, y) => O.compare(x, y) === 1;
}
/**
 * Test whether one value is _non-strictly less than_ another
 *
 * @tsplus getter Ord leq
 */


function leq(O) {
  return (x, y) => O.compare(x, y) !== 1;
}
/**
 * Test whether one value is _strictly less than_ another
 *
 * @tsplus getter Ord lt
 */


function lt_1(O) {
  return (x, y) => O.compare(x, y) === -1;
}
/**
 * Take the maximum of two values. If they are considered equal, the first argument is chosen
 *
 * @tsplus getter Ord max
 */


function max_1(O) {
  return (x, y) => O.compare(x, y) === -1 ? y : x;
}
/**
 * Take the minimum of two values. If they are considered equal, the first argument is chosen
 *
 * @tsplus getter Ord min
 */


function min_1(O) {
  return (x, y) => O.compare(x, y) === 1 ? y : x;
}
/**
 * Test whether a value is between a minimum and a maximum (inclusive)
 *
 * @tsplus getter Ord between
 */


function between(O) {
  const lessThanO = lt_1(O);
  const greaterThanO = gt_1(O);
  return (low, hi) => x => lessThanO(x, low) || greaterThanO(x, hi) ? false : true;
}
/**
 * Clamp a value between a minimum and a maximum
 *
 * @tsplus getter Ord clamp
 */


function clamp(O) {
  const minO = min_1(O);
  const maxO = max_1(O);
  return (low, hi) => x => maxO(minO(x, hi), low);
}
/**
 * Get the dual of an Ord
 *
 * @tsplus getter Ord inverted
 */


function inverted(O) {
  return tsplus_module_1.Ord((x, y) => O.compare(y, x));
}
/**
 * Get an instance of Equal
 *
 * @tsplus getter Ord equivalence
 */


function getEquivalence(O) {
  return tsplus_module_2.Equivalence((x, y) => O.compare(x, y) === 0);
}
//# sourceMappingURL=operations.js.map