import { HKT } from "@tsplus/stdlib/prelude/HKT";
import type { Recursive } from "@tsplus/stdlib/prelude/Recursive";
/**
 * @tsplus type Recursive/Annotated
 * @tsplus companion Recursive/Annotated/Ops
 */
export declare class Annotated<F extends HKT, A, E = unknown> implements Recursive<F> {
    readonly caseValue: HKT.Kind<F, unknown, unknown, Annotated<F, A>>;
    readonly annotations: A;
    constructor(caseValue: HKT.Kind<F, unknown, unknown, Annotated<F, A>>, annotations: A);
}
export declare namespace Annotated {
    /**
     * A function operating on a single level of a recursive structure,
     * with its recursive terms replaced by a value *and* an annotation, which
     * represents the value computed for each child.  aka `Course-of-Value (CV)Algebra`
     */
    type Fn<F extends HKT, Z, E = unknown, R = unknown> = (r: HKT.Kind<F, R, E, Annotated<F, Z>>) => Z;
}
/**
 * @tsplus fluent Recursive/Annotated unfix
 * @tsplus static Recursive/Annotated/Ops unfix
 * @tsplus location "@tsplus/stdlib/prelude/Recursive/Annotated"
 */
export declare function unfixAnnotated<F extends HKT, A>({ caseValue }: Annotated<F, A>): HKT.Kind<F, unknown, unknown, Annotated<F, A>>;
/**
 * @tsplus static Recursive/Annotated/Ops __call
 * @tsplus static Recursive/Annotated/Ops make
 * @tsplus location "@tsplus/stdlib/prelude/Recursive/Annotated"
 */
export declare function makeAnnotated<F extends HKT, Z, E = unknown>(caseValue: HKT.Kind<F, unknown, E, Annotated<F, Z>>, annotations: Z): Annotated<F, Z>;
//# sourceMappingURL=Annotated.d.ts.map