"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.combine = exports._Output = exports._Input = exports.UpdateService = exports.RemoveService = exports.PatchSym = exports.Patch = exports.Empty = exports.BasePatch = exports.AndThen = exports.AddService = void 0;
exports.concretePatch = concretePatch;
exports.diff = diff;
exports.empty = void 0;
exports.patch = patch;

var tsplus_module_1 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/List/from"));

var tsplus_module_2 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/List/definition"));

var tsplus_module_3 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/List/prependAll"));

var _Env = /*#__PURE__*/require("@tsplus/stdlib/service/Env");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

var _a;

const empty = empty_1;
exports.empty = empty;
const combine = combine_1;
exports.combine = combine;
const PatchSym = /*#__PURE__*/Symbol.for("@tsplus/stdlib/service/Patch");
exports.PatchSym = PatchSym;

const _Input = /*#__PURE__*/Symbol.for("@tsplus/stdlib/service/Patch/Input");

exports._Input = _Input;

const _Output = /*#__PURE__*/Symbol.for("@tsplus/stdlib/service/Patch/Output");

exports._Output = _Output;
const Patch = {
  $: {}
};
exports.Patch = Patch;

class BasePatch {
  constructor() {
    this[_a] = PatchSym;
  }

}

exports.BasePatch = BasePatch;
_a = PatchSym;

class Empty extends BasePatch {
  constructor() {
    super();
    this._tag = "Empty";
  }

}

exports.Empty = Empty;

class AddService extends BasePatch {
  constructor(tag, service) {
    super();
    this.tag = tag;
    this.service = service;
    this._tag = "AddService";
  }

}

exports.AddService = AddService;

class AndThen extends BasePatch {
  constructor(first, second) {
    super();
    this.first = first;
    this.second = second;
    this._tag = "AndThen";
  }

}

exports.AndThen = AndThen;

class RemoveService extends BasePatch {
  constructor(tag) {
    super();
    this.tag = tag;
    this._tag = "RemoveService";
  }

}

exports.RemoveService = RemoveService;

class UpdateService extends BasePatch {
  constructor(tag, update) {
    super();
    this.tag = tag;
    this.update = update;
    this._tag = "UpdateService";
  }

}
/**
 * @tsplus macro remove
 */


exports.UpdateService = UpdateService;

function concretePatch(_) {//
}
/**
 * Applies a `Patch` to the specified `Env` to produce a new patched `Env`.
 *
 * @tsplus static Patch.Aspects patch
 * @tsplus pipeable Patch patch
 */


function patch(env) {
  return self => {
    const updatedRef = {
      ref: false
    };
    const updated = patchLoop(new Map(env.unsafeMap), tsplus_module_1.make(self), updatedRef);

    if (!updatedRef.ref) {
      return new _Env.Env(updated);
    }

    const map = new Map();

    for (const [tag] of env.unsafeMap) {
      if (updated.has(tag)) {
        map.set(tag, updated.get(tag));
        updated.delete(tag);
      }
    }

    for (const [tag, s] of updated) {
      map.set(tag, s);
    }

    return new _Env.Env(map);
  };
}
/**
 * @tsplus tailRec
 */


function patchLoop(env, patches, updatedRef) {
  var env_1 = env,
      patches_1 = patches,
      updatedRef_1 = updatedRef;
  var env_2 = env,
      patches_2 = patches,
      updatedRef_2 = updatedRef;

  while (1) {
    if (tsplus_module_2.isNil(patches_1)) {
      return env_1;
    }

    const head = patches_1.head;
    void 0;
    const tail = patches_1.tail;

    switch (head._tag) {
      case "Empty":
        {
          env_2 = env_1;
          patches_2 = tail;
          updatedRef_2 = updatedRef_1;
          env_1 = env_2;
          patches_1 = patches_2;
          updatedRef_1 = updatedRef_2;
          continue;
        }

      case "AddService":
        {
          env_2 = env_1.set(head.tag, head.service);
          patches_2 = tail;
          updatedRef_2 = updatedRef_1;
          env_1 = env_2;
          patches_1 = patches_2;
          updatedRef_1 = updatedRef_2;
          continue;
        }

      case "AndThen":
        {
          env_2 = env_1;
          patches_2 = tsplus_module_3.prependAll(tsplus_module_1.make(head.first, head.second))(tail);
          updatedRef_2 = updatedRef_1;
          env_1 = env_2;
          patches_1 = patches_2;
          updatedRef_1 = updatedRef_2;
          continue;
        }

      case "RemoveService":
        {
          env_2 = (env_1.delete(head.tag), env_1);
          patches_2 = tail;
          updatedRef_2 = updatedRef_1;
          env_1 = env_2;
          patches_1 = patches_2;
          updatedRef_1 = updatedRef_2;
          continue;
        }

      case "UpdateService":
        {
          env_2 = env_1.set(head.tag, head.update(env_1.get(head.tag)));
          patches_2 = tail;
          updatedRef_2 = (updatedRef_1.ref = true, updatedRef_1);
          env_1 = env_2;
          patches_1 = patches_2;
          updatedRef_1 = updatedRef_2;
          continue;
        }
    }
  }
}
/**
 * An empty patch which returns the environment unchanged.
 *
 * @tsplus static Patch.Ops empty
 */


function empty_1() {
  return new Empty();
}
/**
 * Combines two patches to produce a new patch that describes applying the
 * updates from this patch and then the updates from the specified patch.
 *
 * @tsplus static Patch.Aspects combine
 * @tsplus pipeable Patch combine
 */


function combine_1(that) {
  return self => new AndThen(self, that);
}
/**
 * @tsplus static Patch.Ops diff
 */


function diff(oldValue, newValue) {
  const missingServices = new Map(oldValue.unsafeMap);
  let patch = empty_1();

  for (const [tag, newService] of newValue.unsafeMap.entries()) {
    if (missingServices.has(tag)) {
      const old = missingServices.get(tag);
      missingServices.delete(tag);

      if (old !== newService) {
        patch = combine_1(new UpdateService(tag, () => newService))(patch);
      }
    } else {
      missingServices.delete(tag);
      patch = combine_1(new AddService(tag, newService))(patch);
    }
  }

  for (const [tag] of missingServices.entries()) {
    patch = combine_1(new RemoveService(tag))(patch);
  }

  return patch;
}
//# sourceMappingURL=Patch.js.map