"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Hash = void 0;
exports.combine = combine;
exports.hashArgs = hashArgs;
exports.hashArray = void 0;
exports.hashIterator = hashIterator;
exports.hashMap = void 0;
exports.hashMiscRef = hashMiscRef;
exports.hashNumber = hashNumber;
exports.hashObject = hashObject;
exports.hashRandom = exports.hashPlainObject = void 0;
exports.hashRandomCached = hashRandomCached;
exports.hashSet = void 0;
exports.hashString = hashString;
exports.optimize = exports.isHash = exports.hashUnknown = void 0;

var tsplus_module_1 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/io/Random/PCGRandom"));

var tsplus_module_2 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/utilities/Guards"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

const isHash = isHash_1;
exports.isHash = isHash;
const optimize = optimize_1;
exports.optimize = optimize;
const hashUnknown = hashUnknown_1;
exports.hashUnknown = hashUnknown;
const hashArray = hashArray_1;
exports.hashArray = hashArray;
const hashMap = hashMap_1;
exports.hashMap = hashMap;
const hashSet = hashSet_1;
exports.hashSet = hashSet;
const hashPlainObject = hashPlainObject_1;
exports.hashPlainObject = hashPlainObject;
const hashRandom = hashRandom_1;
exports.hashRandom = hashRandom;
const Hash = {
  sym: /*#__PURE__*/Symbol.for("tsplus/Hash")
};
/**
 * @tsplus static Hash.Ops isHash
 */

exports.Hash = Hash;

function isHash_1(u) {
  return typeof u === "object" && u !== null && Hash.sym in u;
}
/**
 * @tsplus static Hash.Ops optimize
 */


function optimize_1(n) {
  return n & 0xbfffffff | n >>> 1 & 0x40000000;
}
/**
 * @tsplus static Hash.Ops unknown
 * @tsplus getter Hash hash
 */


function hashUnknown_1(arg) {
  return optimize_1(_hash(arg));
}
/**
 * @tsplus static Hash.Ops array
 */


function hashArray_1(arr) {
  return optimize_1(_hashArray(arr));
}
/**
 * @tsplus static Hash.Ops map
 */


function hashMap_1(arr) {
  return optimize_1(_hashMap(arr));
}
/**
 * @tsplus static Hash.Ops set
 */


function hashSet_1(arr) {
  return optimize_1(_hashSet(arr));
}

function hashArgs() {
  let h = 5381;

  for (let i = 0; i < arguments.length; i++) {
    // eslint-disable-next-line prefer-rest-params
    h = _combineHash(h, hashUnknown_1(arguments[i]));
  }

  return optimize_1(h);
}
/**
 * @tsplus static Hash.Ops combine
 */


function combine(a, b) {
  return optimize_1(_combineHash(a, b));
}
/**
 * @tsplus static Hash.Ops object
 */


function hashObject(value) {
  return optimize_1(_hashObject(value));
}
/**
 * @tsplus static Hash.Ops miscRef
 */


function hashMiscRef(o) {
  return optimize_1(_hashMiscRef(o));
}
/**
 * @tsplus static Hash.Ops iterator
 */


function hashIterator(it) {
  return optimize_1(_hashIterator(it));
}
/**
 * @tsplus static Hash.Ops plainObject
 */


function hashPlainObject_1(o) {
  return optimize_1(_hashPlainObject(o));
}
/**
 * @tsplus static Hash.Ops number
 */


function hashNumber(n) {
  return optimize_1(_hashNumber(n));
}
/**
 * @tsplus static Hash.Ops string
 */


function hashString(str) {
  return optimize_1(_hashString(str));
}
/**
 * @tsplus static Hash.Ops random
 */


function hashRandom_1() {
  return optimize_1(randomInt());
}
/**
 * @tsplus static Hash.Ops randomCached
 */


function hashRandomCached(o) {
  if (CACHE.has(o)) {
    return CACHE.get(o);
  }

  const h = optimize_1(randomInt());
  CACHE.set(o, h);
  return h;
}

function isZero(value) {
  return value === null || value === void 0 || value === false;
}

const RANDOM = /*#__PURE__*/new tsplus_module_1.PCGRandom( /*#__PURE__*/Math.random() * 4294967296 >>> 0);
const CACHE = /*#__PURE__*/new WeakMap();

function randomInt() {
  return RANDOM.integer(0x7fffffff);
}

function _hash(arg) {
  if (isZero(arg)) return 0;

  if (typeof arg.valueOf === "function" && arg.valueOf !== Object.prototype.valueOf) {
    arg = arg.valueOf();
    if (isZero(arg)) return 0;
  }

  switch (typeof arg) {
    case "number":
      return _hashNumber(arg);

    case "string":
      return _hashString(arg);

    case "function":
      return _hashMiscRef(arg);

    case "object":
      return _hashObject(arg);

    case "boolean":
      return arg === true ? 1 : 0;

    case "symbol":
      return _hashString(String(arg));

    case "bigint":
      return _hashString(arg.toString(10));

    case "undefined":
      {
        return 0;
      }
  }
}

function _hashArray(arr) {
  let h = 6151;

  for (let i = 0; i < arr.length; i++) {
    h = _combineHash(h, _hash(arr[i]));
  }

  return h;
}

function _hashMap(arr) {
  let h = 9744;
  arr.forEach((v, k) => {
    h ^= _combineHash(_hash(k), _hash(v));
  });
  return h;
}

function _hashSet(arr) {
  let h = 2362;
  arr.forEach(v => {
    h ^= _hash(v);
  });
  return h;
}

function _combineHash(a, b) {
  return a * 53 ^ b;
}

const protoMap = /*#__PURE__*/new Map([[Array.prototype, hashArray_1], [Map.prototype, hashMap_1], [Set.prototype, hashSet_1], [Object.prototype, hashPlainObject_1]]);

function _hashObject(value) {
  let h = CACHE.get(value);
  if (tsplus_module_2.isDefined(h)) return h;

  if (isHash_1(value)) {
    h = value[Hash.sym]();
  } else {
    const primitiveHash = protoMap.get(Object.getPrototypeOf(value));

    if (primitiveHash) {
      h = primitiveHash(value);
    } else {
      h = hashRandom_1();
    }
  }

  CACHE.set(value, h);
  return h;
}

function _hashMiscRef(o) {
  let h = CACHE.get(o);
  if (tsplus_module_2.isDefined(h)) return h;
  h = randomInt();
  CACHE.set(o, h);
  return h;
}

function _hashIterator(it) {
  let h = 6151;
  let current;

  while (!(current = it.next()).done) {
    h = _combineHash(h, hashUnknown_1(current.value));
  }

  return h;
}

function _hashPlainObject(o) {
  CACHE.set(o, randomInt());
  const keys = Object.keys(o);
  let h = 12289;

  for (let i = 0; i < keys.length; i++) {
    h ^= _combineHash(_hashString(keys[i]), hashUnknown_1(o[keys[i]]));
  }

  return h;
}

function _hashNumber(n) {
  if (n !== n || n === Infinity) return 0;
  let h = n | 0;
  if (h !== n) h ^= n * 0xffffffff;

  while (n > 0xffffffff) h ^= n /= 0xffffffff;

  return n;
}

function _hashString(str) {
  let h = 5381,
      i = str.length;

  while (i) h = h * 33 ^ str.charCodeAt(--i);

  return h;
}
//# sourceMappingURL=Hash.js.map